"""
Hosts - file ``/etc/hosts``
===========================

This parser parses the ``/etc/hosts`` file, strips the comments, ignores the
blank lines, and collects the host names by IP address.  IPv4 and IPv6
addresses are supported.

Sample hosts file::

    127.0.0.1 localhost localhost.localdomain localhost4 localhost4.localdomain4
    ::1 localhost localhost.localdomain localhost6 localhost6.localdomain6
    # The same IP address can appear more than once, with different names
    127.0.0.1 fte.example.com

    10.0.0.1 nonlocal.example.com nonlocal2.fte.example.com
    10.0.0.2 other.host.example.com # Comments at end of line are ignored

Examples:

    >>> len(hosts.all_names)
    10
    >>> 'localhost6'in  hosts.all_names
    True
    >>> hosts.data['127.0.0.1']
    ['localhost', 'localhost.localdomain', 'localhost4', 'localhost4.localdomain4', 'fte.example.com']
    >>> sorted(hosts.get_nonlocal().keys())
    ['10.0.0.1', '10.0.0.2']
    >>> hosts.lines[-1]['ip']
    '10.0.0.2'
    >>> hosts.lines[2]['names']
    ['fte.example.com']


"""

from insights import Parser, parser
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.hosts)
class Hosts(Parser):
    """
    Read the ``/etc/hosts`` file and parse it into a dictionary of host name
    lists, keyed on IP address.
    """

    def parse_content(self, content):
        self._ips = set()
        self._names = set()
        self._lines = list()
        self._data = dict()
        for line in content:
            line = line.strip()
            if "#" in line:
                line = line.split("#")[0]
            words = line.split(None)
            if len(words) > 1:
                ip, names = words[0], words[1:]
                line_data = {'ip': ip, 'names': names, 'raw_line': line}
                self._ips.add(ip)
                self._names.update(names)
                self._lines.append(line_data)
                if ip not in self._data:
                    self._data[ip] = []
                self._data[ip].extend(names)

        if not self._data:
            raise SkipException("No useful data")

    @property
    def data(self):
        """
        (dict): The parsed result as a dict with IP address as the key.
        """
        return self._data

    @property
    def lines(self):
        """
        (list): List of the parsed lines in the original order, in the following
            format::

                {
                    'ip': '127.0.0.1',
                    'names': ['localhost', 'localhost.localdomain', 'localhost4', 'localhost4.localdomain4']
                    'raw_line:' '127.0.0.1   localhost localhost.localdomain localhost4 localhost4.localdomain4'
                }
        """
        return self._lines

    @property
    def all_ips(self):
        """
        (set): The set of ip addresses known.
        """
        return self._ips

    @property
    def all_names(self):
        """
        (set): The set of host names known, regardless of their IP address.
        """
        return self._names

    def get_nonlocal(self):
        """
        A dictionary of host name lists, keyed on IP address, that are not
        the 'localhost' addresses '127.0.0.1' or '::1'.
        """
        return dict((ip, host_list)
                    for ip, host_list in self._data.items()
                    if ip not in ("127.0.0.1", "::1"))

    def ip_of(self, hostname):
        """
        Return the (first) IP address given for this host name.  None is
        returned if no IP address is found.
        """
        for ip, host_list in self._data.items():
            if hostname in host_list:
                return ip
        return None



"""
EtcUdevRules - file ``/etc/udev/rules.d/``
==========================================

This module is similar to the :py:mod:`insights.parsers.udev_rules`
but parse .rules files under ``/etc/ude/rules.d/`` directory instead.

The parsers included in this module are:

UdevRules40Redhat - file ``/etc/udev/rules.d/40-redhat.rules``
--------------------------------------------------------------
"""
from insights import parser
from insights.core import LogFileOutput
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.etc_udev_40_redhat_rules)
class UdevRules40Redhat(LogFileOutput):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.udev_rules.UdevRules40Redhat` instead.

    Read the content of ``/etc/udev/rules.d/40-redhat.rules`` file.

    .. note::

        The syntax of the `.rules` file is complex, and no rules require to
        get the serialized parsed result currently.  An only existing rule's
        supposed to check the syntax of some specific line, so here the
        :class:`insights.core.LogFileOutput` is the base class.

    Sample input::

        # do not edit this file, it will be overwritten on update
        # CPU hotadd request
        SUBSYSTEM=="cpu", ACTION=="add", TEST=="online", ATTR{online}=="0", ATTR{online}="1"

        # Memory hotadd request
        SUBSYSTEM!="memory", ACTION!="add", GOTO="memory_hotplug_end"
        PROGRAM="/bin/uname -p", RESULT=="s390*", GOTO="memory_hotplug_end"

        LABEL="memory_hotplug_end"

    Examples:
        >>> 'LABEL="memory_hotplug_end"' in udev_rules.lines
        True
    """
    def __init__(self, *args, **kwargs):
        deprecated(UdevRules40Redhat, "Import UdevRules40Redhat from insights.parsers.udev_rules instread.", "3.0.300")
        super(UdevRules40Redhat, self).__init__(*args, **kwargs)



'''
NetConsole - file ``/etc/sysconfig/netconsole``
===============================================

This parser reads the ``/etc/sysconfig/netconsole`` file.  It uses the
``SysconfigOptions`` parser class to convert the file into a dictionary of
options.

Sample data::

    # This is the configuration file for the netconsole service.  By starting
    # this service you allow a remote syslog daemon to record console output
    # from this system.

    # The local port number that the netconsole module will use
    LOCALPORT=6666


Examples:

    >>> config = shared[NetConsole]
    >>> 'LOCALPORT' in config.data
    True
    >>> 'DEV' in config # Direct access to options
    False

'''
from insights.util import deprecated
from .. import parser, SysconfigOptions, LegacyItemAccess
from insights.specs import Specs


@parser(Specs.netconsole)
class NetConsole(SysconfigOptions, LegacyItemAccess):
    '''
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.sysconfig.NetconsoleSysconfig` instead.

    Contents of the ``/etc/sysconfig/netconsole`` file.  Uses the
    ``SysconfigOptions`` shared parser class.
    '''
    def __init__(self, *args, **kwargs):
        deprecated(NetConsole, "Import NetconsoleSysconfig from insights.parsers.sysconfig instead.", "3.0.300")
        super(NetConsole, self).__init__(*args, **kwargs)



import doctest
import pytest
from insights.parsers import sockstat
from insights.parsers.sockstat import SockStats
from insights.parsers import SkipException
from insights.tests import context_wrap


SOCK_STATS = """
sockets: used 3037
TCP: inuse 1365 orphan 17 tw 2030 alloc 2788 mem 4109
UDP: inuse 6 mem 3
UDPLITE: inuse 0
RAW: inuse 0
FRAG: inuse 0 memory 10
""".strip()

SOCK_STATS_NO = """
""".strip()

SOCK_STATS_DOC = """
sockets: used 3037
TCP: inuse 1365 orphan 17 tw 2030 alloc 2788 mem 4109
UDP: inuse 6 mem 3
UDPLITE: inuse 0
RAW: inuse 0
FRAG: inuse 0 memory 0
""".strip()

SOCK_STATS_NO_2 = """
In valid data is present
""".strip()


def test_sockstat():
    stats = SockStats(context_wrap(SOCK_STATS))
    assert stats.seg_details('sockets') == {'used': '3037'}
    assert stats.seg_details('tcp') == {'inuse': '1365', 'orphan': '17', 'tw': '2030', 'alloc': '2788', 'mem': '4109'}
    assert stats.seg_element_details('tcp', 'inuse') == 1365
    assert stats.seg_element_details('frag', 'memory') == 10
    assert stats.seg_element_details('xyz', 'abc') is None
    assert stats.seg_element_details(None, None) is None
    assert stats.seg_element_details('tcp', 'abc') is None
    assert len(stats.sock_stats)
    with pytest.raises(SkipException) as exc:
        sock_obj = SockStats(context_wrap(SOCK_STATS_NO))
        assert sock_obj is not None
    assert 'No Contents' in str(exc)


def test_modinfo_doc_examples():
    env = {'sock_obj': SockStats(context_wrap(SOCK_STATS_DOC))}
    failed, total = doctest.testmod(sockstat, globs=env)
    assert failed == 0



"""
Custom datasource related PCP (Performance Co-Pilot)
"""
import logging
import datetime
import os

from insights.core.dr import SkipComponent
from insights.core.context import HostContext
from insights.core.plugins import datasource
from insights.parsers.ros_config import RosConfig
from insights.combiners.ps import Ps
from insights.combiners.services import Services

logger = logging.getLogger(__name__)


@datasource(Services, HostContext)
def pcp_enabled(broker):
    """
    Returns:
        bool: True if pmproxy service is on in services

    Raises:
        SkipComponent: When pmproxy service is not enabled
    """
    if not broker[Services].is_on("pmproxy"):
        raise SkipComponent("pmproxy not enabled")
    return True


@datasource(Ps, RosConfig, HostContext)
def pmlog_summary_args(broker):
    """
    Determines the pmlogger file and the metrics to collect via `pmlog_summary`
    spec.

    Returns:
        str: Full arguments string that will be passed to the `pmlogsummary`,
             which contains the `pmlogger` archive file and the required `metrics`.

    Raises:
        SkipComponent: Raises when meeting one of the following scenario:
                       - No pmlogger process is running
                       - No pmlogger file
                       - No "mandatory on" metrics in `config.ros`
    """
    pm_file = None
    try:
        ps = broker[Ps]
        if not ps.search(COMMAND_NAME__contains='pmlogger'):
            raise SkipComponent("No 'pmlogger' is running")

        pcp_log_date = (datetime.date.today() - datetime.timedelta(days=1)).strftime("%Y%m%d")
        pm_file = "/var/log/pcp/pmlogger/ros/{0}.index".format(pcp_log_date)

        if not (os.path.exists(pm_file) and os.path.isfile(pm_file)):
            raise SkipComponent("No pmlogger archive file: {0}".format(pm_file))

    except Exception as e:
        raise SkipComponent("Failed to check pmlogger file existence: {0}".format(str(e)))

    metrics = set()
    try:
        ros = broker[RosConfig]
        for spec in ros.specs:
            if spec.get('state') == 'mandatory on':
                metrics.update(spec.get('metrics').keys())

        if not metrics:
            raise SkipComponent("No 'mandatory on' metrics in config.ros")

    except Exception as e:
        raise SkipComponent("Failed to get pmlogger metrics: {0}".format(str(e)))

    return "{0} {1}".format(pm_file, ' '.join(sorted(metrics)))



from insights.parsers.vsftpd import VsftpdConf, VsftpdPamConf
from insights.tests import context_wrap

VSFTPD_PAM_CONF = """
#%PAM-1.0
session    optional     pam_keyinit.so    force revoke
auth       required     pam_listfile.so item=user sense=deny file=/etc/vsftpd/ftpusers onerr=succeed
auth       required     pam_shells.so
auth       include      password-auth
account    include      password-auth
session    required     pam_loginuid.so
session    include      password-auth
""".strip()

VSFTPD_CONF = """
# No anonymous login
anonymous_enable=NO
# Let local users login
local_enable=YES

# Write permissions
write_enable=YES
# Commented_option=not_present
""".strip()


def test_vsftpd_pam_conf():
    pam_conf = VsftpdPamConf(context_wrap(VSFTPD_PAM_CONF, path='etc/pamd.d/vsftpd'))
    assert len(pam_conf) == 7
    assert pam_conf[0].service == 'vsftpd'
    assert pam_conf[0].interface == 'session'
    assert len(pam_conf[0].control_flags) == 1
    assert pam_conf[0].control_flags[0].flag == 'optional'
    assert pam_conf[0].module_name == 'pam_keyinit.so'
    assert pam_conf[0].module_args == 'force revoke'
    assert pam_conf[1].module_args == 'item=user sense=deny file=/etc/vsftpd/ftpusers onerr=succeed'
    assert pam_conf[6].interface == 'session'
    assert len(pam_conf[6].control_flags) == 1
    assert pam_conf[6].control_flags[0].flag == 'include'
    assert pam_conf[6].module_name == 'password-auth'
    assert pam_conf[6].module_args is None


def test_vsftpd_conf():
    vsftpd_conf = VsftpdConf(context_wrap(VSFTPD_CONF))
    assert vsftpd_conf.get('anonymous_enable') == 'NO'
    assert vsftpd_conf.get('local_enable') == 'YES'
    assert vsftpd_conf.get('write_enable') == 'YES'
    assert 'Commented_option' not in vsftpd_conf



"""
Sap
===
This combiner combines the result of
insights.parsers.saphostctrl.SAPHostCtrlInstances` and
`:class:`insights.parsers.lssap.Lssap` to get the available SAP instances.

Prefer the ``SAPHostCtrlInstances`` to ``Lssap``.
"""

from collections import namedtuple
from insights import SkipComponent
from insights.core.plugins import combiner
from insights.combiners.hostname import Hostname
from insights.parsers.lssap import Lssap
from insights.parsers.saphostctrl import SAPHostCtrlInstances


SAPInstances = namedtuple("SAPInstances",
        field_names=["name", "hostname", "sid", "type", "full_type", "number", "fqdn", "version"])
"""namedtuple: Type for storing the SAP instance."""

FUNC_FULL_TYPES = [
    'Solution Manager Diagnostic Agent',
    'Diagnostic Agent'
]
NETW_TYPES = ('D', 'ASCS', 'DVEBMGS', 'J', 'SCS', 'ERS', 'W', 'G', 'JC')
"""
D      :    NetWeaver (ABAP Dialog Instance)
ASCS   :    NetWeaver (ABAP Central Services)
DVEBMGS:    NetWeaver (Primary Application server)
J      :    NetWeaver (Java App Server Instance)
SCS    :    NetWeaver (Java Central Services)
ERS    :    NetWeaver (Enqueue Replication Server)
W      :    NetWeaver (WebDispatcher)
G      :    NetWeaver (Gateway)
JC     :    NetWeaver (Java App Server Instance)
"""


@combiner(Hostname, [SAPHostCtrlInstances, Lssap])
class Sap(dict):
    """
    Combiner for combining the result of :class:`insights.parsers.lssap.Lssap`
    generated by command ``lssap`` and
    `insights.parsers.saphostctrl.SAPHostCtrlInstances` generated by command
    ``saphostctrl``.

    Prefer ``SAPHostCtrlInstances`` to ``Lssap``.

    Examples:
        >>> type(saps)
        <class 'insights.combiners.sap.Sap'>
        >>> 'D16' in saps
        True
        >>> saps['D16'].number
        '16'
        >>> saps.sid('HDB16')
        'HA2'
        >>> saps.hostname('HDB16')
        'lu0417'
        >>> len(saps.business_instances)
        3
        >>> saps.is_hana
        True
        >>> saps.is_netweaver
        True
        >>> saps.is_ascs
        False

    Attributes:
        all_instances (list): List of all the SAP instances listed by the command.
        function_instances (list): List of functional SAP instances
                                   E.g. Diagnostics Agents SMDA97/SMDA98
        business_instances (list): List of business SAP instances
                                   E.g. HANA, NetWeaver, ASCS, or others
        local_instances (list): List of all SAP instances running on this host
    """
    """ tuple: Tuple of the prefix string of the functional SAP instances"""

    def __init__(self, hostname, insts, lssap):
        hn = hostname.hostname
        fqdn = hostname.fqdn
        data = {}
        self.local_instances = []
        self.business_instances = []
        self.function_instances = []
        self.all_instances = []
        self._types = set()
        if insts:
            self._types = insts.types
            self.all_instances = insts.instances
            for inst in insts:
                k = inst['InstanceName']
                if (hn == inst['Hostname'].split('.')[0] or
                        fqdn == inst['FullQualifiedHostname'] or
                        fqdn == inst['Hostname']):
                    self.local_instances.append(k)
                data[k] = SAPInstances(k,
                                       inst['Hostname'],
                                       inst['SID'],
                                       inst['InstanceName'].strip('1234567890'),
                                       inst['InstanceType'],
                                       inst['SystemNumber'],
                                       inst['FullQualifiedHostname'],
                                       inst['SapVersionInfo'])
        elif lssap:
            for inst in lssap.data:
                k = inst['Instance']
                t = k.rstrip('1234567890')
                self.all_instances.append(k)
                self._types.add(t)
                self.local_instances.append(k) if hn == inst['SAPLOCALHOST'].split('.')[0] else None
                data[k] = SAPInstances(k,
                                       inst['SAPLOCALHOST'],
                                       inst['SID'],
                                       t,
                                       t,  # Use short
                                       inst['Nr'],
                                       None,
                                       inst['Version'])
        if not data:
            raise SkipComponent('No SAP instance.')

        self.update(data)

        for i in self.values():
            (self.function_instances
                if i.full_type in FUNC_FULL_TYPES else
                    self.business_instances).append(i.name)

    def version(self, instance):
        """str: Returns the version of the ``instance``."""
        return self[instance].version if instance in self else None

    def sid(self, instance):
        """str: Returns the sid of the ``instance``."""
        return self[instance].sid if instance in self else None

    def type(self, instance):
        """str: Returns the short type code of the ``instance``."""
        return self[instance].type if instance in self else None

    def full_type(self, instance):
        """str: Returns the full type code of the ``instance``."""
        return self[instance].full_type if instance in self else None

    def hostname(self, instance):
        """str: Returns the hostname of the ``instance``."""
        return self[instance].hostname if instance in self else None

    def number(self, instance):
        """str: Returns the systeme number of the ``instance``."""
        return self[instance].number if instance in self else None

    @property
    def is_netweaver(self):
        """bool: Is any SAP NetWeaver instance detected?"""
        return any(_t in self._types for _t in NETW_TYPES)

    @property
    def is_hana(self):
        """bool: Is any SAP HANA instance detected?"""
        return 'HDB' in self._types

    @property
    def is_ascs(self):
        """bool: Is any ABAP Central Services instance detected?"""
        return 'ASCS' in self._types

    @property
    def data(self):
        """dict: Dict with the instance name as the key and instance details as the value."""
        return self



from insights import condition, rule, make_fail
from insights.parsers.installed_rpms import InstalledRpms
from insights.parsers.up2date import Up2Date
from insights.parsers.uname import Uname
from insights.parsers.redhat_release import RedhatRelease
from insights.parsers.messages import Messages
from insights.parsers.ps import PsAuxcww
from insights.parsers.virt_what import VirtWhat
from insights.parsers.date import Date
from insights.parsers.tags import Tags
from insights.parsers.lspci import LsPci, LsPciVmmkn
from insights.parsers.lsof import Lsof
from insights.parsers.lscpu import LsCPU
from insights.parsers.lsblk import LSBlock
from insights.parsers.ls_etc import LsEtc
from insights.parsers.ls_boot import LsBoot
from insights.parsers.ls_dev import LsDev
from insights.parsers.ls_disk import LsDisk
from insights.parsers.ls_tmp import LsTmp
from insights.parsers.ls_usr_bin import LsUsrBin
from insights.parsers.ls_var_log import LsVarLog
from insights.parsers.ls_var_tmp import LsVarTmp
from insights.parsers.ls_var_www_perms import LsVarWwwPerms
from insights.parsers.ls_var_run import LsVarRun
from insights.parsers.mount import Mount
from insights.parsers.lvm import LvmConfig, LvmConf
from insights.parsers.lsscsi import LsSCSI
from insights.parsers.uptime import Uptime
from insights.combiners.redhat_release import RedHatRelease
from insights.core.dr import get_dependency_specs


@condition(InstalledRpms, [Up2Date, PsAuxcww], optional=[Messages])
def condition_01(*args):
    return True


@condition(VirtWhat, Tags, RedHatRelease, optional=[Date, Uptime])
def condition_02(*args):
    return True


@condition(LsVarLog, [LsBoot, LsEtc])
def condition_03(*args):
    return True


@condition(LsVarWwwPerms, [LsDev, LsDisk], optional=[LsVarRun])
def condition_04(*args):
    return True


@condition([LsPci, LsPciVmmkn])
def condition_05(*args):
    return True


@condition([LvmConf, LvmConfig])
def condition_06(*args):
    return True


@condition(Mount, [LsTmp, LsVarTmp], optional=[LsUsrBin])
def condition_07(*args):
    return True


@rule(Lsof, condition_01, condition_02,
      [LsCPU, condition_03], [condition_04, LsSCSI],
      [condition_05, condition_06],
      optional=[LSBlock, condition_07])
def report_01(*args):
    return make_fail("HIT")


@condition(LsVarLog, [LsBoot, LsEtc])
def condition_11(*args):
    return True


@condition([LsPci, condition_11])
def condition_12(*args):
    return True


@condition(LsPci, LsBoot, LsDisk)
def report_11(*args):
    return True


@condition(PsAuxcww, LsVarLog, [Uname, RedhatRelease], [LsPci, condition_11])
def condition_21(*args):
    return True


@condition(RedHatRelease, LsVarLog, condition_12, condition_21)
def report_21(*args):
    return True


def test_get_dependency_specs_1_level_requires_only():
    specs = get_dependency_specs(report_11)
    assert sorted(specs) == ['ls_boot', 'ls_disk', 'lspci']


def test_get_dependency_specs_2_level():
    specs = get_dependency_specs(condition_12)
    # [
    #     ('lspci', [('ls_etc', 'ls_boot'), 'ls_var_log'])
    # ]
    assert len([alo for alo in specs if isinstance(alo, tuple)]) == 1
    assert 'lspci' in specs[0]
    x, y = specs[0]
    if x == 'lspci':
        assert 'ls_var_log' in y
    else:
        assert 'ls_var_log' in x
        m, n = x
        if isinstance(m, tuple):
            t = m
        else:
            t = n
        assert all(i in t for i in ('ls_etc', 'ls_boot'))


def test_get_dependency_specs_complex():
    specs = get_dependency_specs(report_01)
    # [
    #     'installed_rpms',
    #     'tags',
    #     'lsof',
    #     'virt_what',
    #     ('redhat_release', 'uname'),
    #     ('ps_auxcww', 'up2date'),
    #     ('ls_cpu', ['ls_var_log', ('ls_boot', 'ls_etc')]),
    #     ('lsscsi', ['ls_var_tmp', ('ls_disk', 'ls_dev')]),
    #     (('lspci', 'lspci_vmmkn'), ('lvm_conf', 'lvmconfig'))
    # ]
    requires = ['installed_rpms', 'tags', 'lsof', 'virt_what']
    assert all(req in specs for req in requires) is True
    # It's difficult to check the details since the result would in different
    # order due to python versions.  Here we just check the number of the
    # `at_least_one` specs
    assert len([alo for alo in specs if isinstance(alo, tuple)]) == 5


def test_get_dependency_specs_duplicate():
    specs = get_dependency_specs(report_21)
    # [
    #     ('uname', 'redhat_release'),
    #     'ps_auxcww',
    #     'ls_var_log',
    #     ('ls_pci', ['ls_var_log', ('ls_boot', 'ls_etc')])
    # ]
    # There is only one such item in the result
    assert ('uname', 'redhat_release') in specs
    specs.remove(('uname', 'redhat_release'))
    assert ('uname', 'redhat_release') not in specs

    assert 'ls_var_log' in specs
    specs.remove('ls_var_log')
    assert 'ls_var_log' not in specs



from insights.parsers import sap_host_profile, SkipException
from insights.parsers.sap_host_profile import SAPHostProfile
from insights.tests import context_wrap
import doctest
import pytest

HOST_PROFILE_DOC = """
SAPSYSTEMNAME = SAP
SAPSYSTEM = 99
service/porttypes = SAPHostControl SAPOscol SAPCCMS
DIR_LIBRARY =
DIR_EXECUTABLE = /usr/sap/hostctrl/exe
DIR_PROFILE = /usr/sap/hostctrl/exe
DIR_GLOBAL = /usr/sap/hostctrl/exe
DIR_INSTANCE = /usr/sap/hostctrl/exe
DIR_HOME = /usr/sap/hostctrl/work
""".strip()

HOST_PROFILE_AB = """
SAPSYSTEMNAME = SAP
SAPSYSTEM = 99
service/porttypes = SAPHostControl SAPOscol SAPCCMS
DIR_LIBRARY = /usr/sap/hostctrl/exe
DIR_EXECUTABLE = /usr/sap/hostctrl/exe
DIR_PROFILE = /usr/sap/hostctrl/exe
DIR_GLOBAL
""".strip()


def test_sap_host_profile():
    hpf = SAPHostProfile(context_wrap(HOST_PROFILE_DOC))
    assert "SAPSYSTEM" in hpf
    assert hpf["DIR_GLOBAL"] == "/usr/sap/hostctrl/exe"
    assert hpf["DIR_LIBRARY"] == ""


def test_sap_host_profile_abnormal():
    with pytest.raises(SkipException) as s:
        SAPHostProfile(context_wrap(HOST_PROFILE_AB))
    assert "Incorrect line: 'DIR_GLOBAL'" in str(s)


def test_doc_examples():
    env = {
            'hpf': SAPHostProfile(context_wrap(HOST_PROFILE_DOC)),
          }
    failed, total = doctest.testmod(sap_host_profile, globs=env)
    assert failed == 0



import pytest
import doctest
from insights.parsers import gcp_instance_type
from insights.parsers.gcp_instance_type import GCPInstanceType
from insights.tests import context_wrap
from insights.parsers import SkipException, ParseException

GOOGLE_TYPE_1 = "projects/123456789/machineTypes/n2-highcpu-16"
GOOGLE_TYPE_2 = "projects/123456789/machineTypes/e2-medium"
GOOGLE_TYPE_3 = """
 % Total    % Received % Xferd  Average Speed   Time    Time     Time  Current
                                  Dload  Upload   Total   Spent    Left  Speed
   0     0    0     0    0     0      0      0 --:--:-- --:--:-- --:--:--     0
 100  1126  100  1126    0     0  1374k      0 --:--:-- --:--:-- --:--:-- 1099k
projects/123456789/machineTypes/e2-medium
"""
GOOGLE_TYPE_DOC = GOOGLE_TYPE_1
GOOGLE_TYPE_AB_1 = """
curl: (7) Failed to connect to 169.254.169.254 port 80: Connection timed out
""".strip()
GOOGLE_TYPE_AB_2 = """
curl: (7) couldn't connect to host
""".strip()
GOOGLE_TYPE_AB_3 = """
curl: (28) connect() timed out!
""".strip()
GOOGLE_TYPE_AB_4 = """
.micro
""".strip()


def test_gcp_instance_type_ab_other():
    with pytest.raises(SkipException):
        GCPInstanceType(context_wrap(GOOGLE_TYPE_AB_1))

    with pytest.raises(SkipException):
        GCPInstanceType(context_wrap(GOOGLE_TYPE_AB_2))

    with pytest.raises(SkipException):
        GCPInstanceType(context_wrap(GOOGLE_TYPE_AB_3))

    with pytest.raises(ParseException) as pe:
        GCPInstanceType(context_wrap(GOOGLE_TYPE_AB_4))
        assert 'Unrecognized type' in str(pe)


def test_gcp_instance_type_ab_empty():
    with pytest.raises(SkipException):
        GCPInstanceType(context_wrap(''))


def test_gcp_instance_type():
    google = GCPInstanceType(context_wrap(GOOGLE_TYPE_1))
    assert google.type == "n2"
    assert google.size == "highcpu-16"
    assert google.raw == "n2-highcpu-16"
    assert google.raw_line == GOOGLE_TYPE_1

    google = GCPInstanceType(context_wrap(GOOGLE_TYPE_2))
    assert google.type == "e2"
    assert google.size == "medium"
    assert google.raw == "e2-medium"
    assert google.raw_line == GOOGLE_TYPE_2
    assert "e2-medium" in str(google)


def test_gcp_instance_type_stats():
    google = GCPInstanceType(context_wrap(GOOGLE_TYPE_3))
    assert google.type == "e2"
    assert google.size == "medium"
    assert google.raw == "e2-medium"
    assert google.raw_line == GOOGLE_TYPE_2
    assert "e2-medium" in str(google)


def test_doc_examples():
    env = {
            'gcp_inst': GCPInstanceType(context_wrap(GOOGLE_TYPE_DOC))
          }
    failed, total = doctest.testmod(gcp_instance_type, globs=env)
    assert failed == 0



"""
Custom datasources for cloud initialization information
"""
import yaml

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.filters import get_filters
from insights.core.spec_factory import DatasourceProvider, simple_file
from insights.specs import Specs


class LocalSpecs(Specs):
    """ Local specs used only by cloud_init datasources """

    cloud_cfg_input = simple_file("/etc/cloud/cloud.cfg")
    """ Returns the contents of the file ``/etc/cloud/cloud.cfg`` """


@datasource(LocalSpecs.cloud_cfg_input, HostContext)
def cloud_cfg(broker):
    """
    This datasource provides configuration of ``/etc/cloud/cloud.cfg`` file.

    .. note::
        Since this file may contain sensitive information, it should be
        filtered before the Insights collecting it.  The filters will be added
        via the :mod:`insights.specs.Specs.cloud_cfg` Spec.  If nothing is
        added to the filter, nothing will be collected.

    Typical content of ``/etc/cloud/cloud.cfg`` file is::

        #cloud-config
        users:
          - name: demo
            ssh-authorized-keys:
              - key_one
              - key_two
            passwd: $6$j212wezy$7H/1LT4f9/N3wpgNunhsIqtMj62OKiS3nyNwuizouQc3u7

        ssh_deletekeys: 1

        network:
            version: 1
            config:
              - type: physical
                name: eth0
                subnets:
                  - type: dhcp
                  - type: dhcp6

        system_info:
            default_user:
            name: user2
            plain_text_passwd: 'someP@assword'
            home: /home/user2

        debug:
            output: /var/log/cloud-init-debug.log
            verbose: true

    Returns:
        str: YAML string after removing the sensitive information.

    Raises:
        SkipComponent: When the path does not exist, nothing is collected,
                       or any exception occurs.
    """
    relative_path = '/etc/cloud/cloud.cfg'
    try:
        filters = get_filters(Specs.cloud_cfg)
        content = broker[LocalSpecs.cloud_cfg_input].content
        if content and filters:
            result = dict()
            content = yaml.load('\n'.join(content), Loader=yaml.SafeLoader)
            if isinstance(content, dict):
                # apply filters after ignoring sensitive data
                for item in filters:
                    if item not in ('users', 'system_info') and item in content:
                        result[item] = content[item]

                if result:
                    return DatasourceProvider(content=yaml.dump(result), relative_path=relative_path)
            raise SkipComponent("Invalid YAML format")
    except Exception as e:
        raise SkipComponent("Unexpected exception:{e}".format(e=str(e)))
    raise SkipComponent



"""
YumListInstalled - Command ``yum list installed``
=================================================

.. warning::
    This module is deprecated, please import the parsers in
    :py:mod:`insights.parsers.yum_list` instead.
"""
from collections import defaultdict

from insights import CommandParser, parser, SkipComponent
from insights.specs import Specs
from insights.parsers.installed_rpms import InstalledRpm, RpmList
from insights.util import deprecated


class YumInstalledRpm(InstalledRpm):
    """
    The same as :py:class:`insights.parsers.installed_rpms.InstalledRpm` but
    with an additional ``.repo`` attribute.
    """
    def __init__(self, data):
        deprecated(YumInstalledRpm, "Import YumListRpm from insights.parsers.yum_list instead.", "3.0.300")
        self.repo = None
        """str: yum / dnf repository name, if available."""

        super(YumInstalledRpm, self).__init__(data)


@parser(Specs.yum_list_installed)
class YumListInstalled(CommandParser, RpmList):
    """
    ``YumListInstalled`` shares the :py:class:`insights.parsers.installed_rpms.RpmList` interface with
    :py:class:`insights.parsers.installed_rpms.InstalledRpms`. The only difference is ``YumListInstalled``
    takes the output of ``yum list installed`` as its source data, and the
    :py:class:`YumInstalledRpm` instances it produces contain a ``.repo``
    attribute.
    """

    def __init__(self, context):
        deprecated(YumListInstalled, "Import YumListBase from insights.parsers.yum_list instead.", "3.0.300")
        self.expired_cache = False
        """bool: Indicates if the yum repo cache is expired."""

        super(YumListInstalled, self).__init__(context)

    def _find_start(self, content):
        for i, c in enumerate(content):
            if 'Repodata is over 2 weeks old' in c:
                self.expired_cache = True
            elif c == "Installed Packages":
                break
        return i + 1

    def _get_rows(self, content):
        """
        Yields:
            a list per row of the following form:

            .. code:: python

                [
                    <name.arch>,
                    <[epoch:]version-release>,
                    <repo or @installed-from-repo>
                ]
        """
        start = self._find_start(content)
        if start == len(content):
            raise SkipComponent()

        # join hanging wrapped lines together into a single line.
        # see https://bugzilla.redhat.com/show_bug.cgi?id=584525
        cur = []
        for line in content[start:]:
            if not cur:
                cur.append(line.strip())
            elif line.startswith(" "):
                cur.append(line.strip())
            else:
                yield " ".join(cur).split()
                cur = [line.strip()]

        if cur:
            yield " ".join(cur).split()

    def _make_record(self, package, ver_rel, repo):
        """
        Given the fields of a ``yum list installed`` row, return a dictionary
        of name, version, release, epoch, arch, and repo.
        """
        name, _, arch = package.rpartition(".")
        repo = repo.lstrip("@")

        # Kept as string in InstalledRpm. Duplicating here for consistency.
        epoch = "0"
        if ":" in ver_rel:
            epoch, ver_rel = ver_rel.split(":", 1)
        version, release = ver_rel.split("-")

        # This is special cased for InstalledRpm. Duplicating here for
        # consistency.
        if name.startswith('oracleasm') and name.endswith('.el5'):
            name, version2 = name.split('-', 1)
            version = version2 + '-' + version

        return {"name": name, "version": version, "release": release, "epoch":
                epoch, "arch": arch, "repo": repo}

    def _unknown_row(self, row):
        """
        Heuristic to tell us we've hit the bottom of the Installed
        Packages stanza.
        """
        return len(row) != 3 or row[:2] == ["Loaded", "plugins:"]

    def parse_content(self, content):
        """
        ``yum list installed`` output is basically tabular with an ignorable
        set of rows at the top and a line "Installed Packages" that designates
        the following rows as data. Each column has a maximum width, and if any
        column overflows, the following columns wrap to the next line and
        indent to their usual starting positions. It's also possible for the
        data rows to be followed by more lines that should be ignored. Since
        ``yum list installed`` is for human consumption, the footer lines can be
        syntactically ambiguous with data lines. We use heuristics to check for
        an invalid row to signal the end of data.
        """

        packages = defaultdict(list)
        for row in self._get_rows(content):
            if self._unknown_row(row):
                break
            rec = self._make_record(*row)
            packages[rec["name"]].append(YumInstalledRpm(rec))
        self.packages = dict(packages)



"""
Combiners for the parsers which parse the output of ``modinfo <module_name>``
=============================================================================

ModInfo
-------
The ModInfo combiner gathers all the ModInfoEach parsers into a dictionary
indexed by the module name.

ModulesInfo
-----------
The ModulesInfo combines the collected modules info.
It combines the result of ``KernelModulesInfo``, ``ModInfoI40e``,
``ModInfoIgb``, ``ModInfoIxgbe``, ``ModInfoVeth``, ``ModInfoVmxnet3``
if they exists.
"""

from insights.core.plugins import combiner
from insights.parsers import SkipException
from insights.parsers.modinfo import ModInfoEach, ModInfoAll
from insights.parsers.modinfo import (
    KernelModulesInfo, ModInfoI40e, ModInfoIgb, ModInfoIxgbe, ModInfoVeth,
    ModInfoVmxnet3
)
from insights import SkipComponent
from insights.util import deprecated


@combiner([ModInfoAll, ModInfoEach])
class ModInfo(dict):
    """
    .. warning::
        This combiner is deprecated, please use
        :py:class:`insights.combiners.modinfo.ModulesInfo` instead.

    Combiner for accessing all the modinfo outputs.

    Examples:
        >>> type(modinfo_obj)
        <class 'insights.combiners.modinfo.ModInfo'>
        >>> type(modinfo_obj['i40e'])
        <class 'insights.parsers.modinfo.ModInfoEach'>
        >>> modinfo_obj['i40e'].module_name
        'i40e'
        >>> modinfo_obj['i40e'].module_name
        'i40e'
        >>> modinfo_obj['i40e']['retpoline']
        'Y'
        >>> modinfo_obj['i40e'].module_version
        '2.3.2-k'
        >>> modinfo_obj['i40e'].module_path
        '/lib/modules/3.10.0-993.el7.x86_64/kernel/drivers/net/ethernet/intel/i40e/i40e.ko.xz'
        >>> "i40e" in modinfo_obj.retpoline_y
        True
        >>> "bnx2x" in modinfo_obj.retpoline_y
        False
        >>> "bnx2x" in modinfo_obj.retpoline_n
        True

    Raises:
        SkipComponent: When content is empty.

    Attributes:
        retpoline_y (set): A set of names of the modules with the attribute "retpoline: Y".
        retpoline_n (set): A set of names of the modules with the attribute "retpoline: N".
    """
    def __init__(self, mi_all, mi_each):
        deprecated(
            ModInfo,
            "Please use the :class:`insights.parsers.modinfo.KernelModulesInfo` instead.",
            "3.1.25"
        )
        self.retpoline_y = set()
        self.retpoline_n = set()
        if mi_all:
            self.update(mi_all)
            self.retpoline_y = mi_all.retpoline_y
            self.retpoline_n = mi_all.retpoline_n
        else:
            for m in mi_each:
                name = m.module_name
                self[name] = m
                self.retpoline_y.add(name) if m.get('retpoline') == 'Y' else None
                self.retpoline_n.add(name) if m.get('retpoline') == 'N' else None

        if len(self) == 0:
            raise SkipComponent("No Parsed Contents")

    @property
    def data(self):
        """
        (dict): Dict with the module name as the key and the module details as the value.
        """
        return self


@combiner(
    [
        KernelModulesInfo, ModInfoI40e, ModInfoIgb, ModInfoIxgbe, ModInfoVeth,
        ModInfoVmxnet3
    ])
class ModulesInfo(dict):
    """
    Combiner to combine the result of KernelModulesInfo which supports filter
    and the parsers which only support one single module. It refers
    ``KernelModulesInfo`` first.

    Examples:
        >>> type(modules_obj)
        <class 'insights.combiners.modinfo.ModulesInfo'>
        >>> 'i40e' in modules_obj
        True
        >>> 'bnx2x' in modules_obj.retpoline_y
        False

    Raises:
        SkipException: When content is empty.

    Attributes:
        retpoline_y (set): A set of names of the modules with the attribute "retpoline: Y".
        retpoline_n (set): A set of names of the modules with the attribute "retpoline: N".
    """
    def __init__(self, filtered_modules_info, i40e_info, igb_info, ixgbe_info, veth_info, vmxnet3_info):
        self.retpoline_y = set()
        self.retpoline_n = set()
        if filtered_modules_info:
            self.update(filtered_modules_info)
            self.retpoline_n = filtered_modules_info.retpoline_n
            self.retpoline_y = filtered_modules_info.retpoline_y
        for item in filter(None, [i40e_info, igb_info, ixgbe_info, veth_info, vmxnet3_info]):
            name = item.module_name
            self[name] = item
            self.retpoline_y.add(name) if item.get('retpoline') == 'Y' else None
            self.retpoline_n.add(name) if item.get('retpoline') == 'N' else None
        if not self:
            raise SkipException



import pytest
import json
from mock.mock import Mock
from insights import dr
from insights.core import filters
from insights.core.dr import SkipComponent
from insights.core.spec_factory import DatasourceProvider
from insights.specs import Specs
from insights.specs.datasources.container.containers_inspect import containers_inspect_data_datasource, LocalSpecs, running_rhel_containers_id
from insights.specs.datasources.container import running_rhel_containers


INSPECT_1 = """
[
    {
        "Id": "aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8",
        "Created": "2022-10-21T23:47:24.506159696-04:00",
        "Path": "sleep",
        "Args": [
            "1000000"
        ],
        "State": {
            "OciVersion": "1.0.2-dev",
            "Status": "running",
            "Running": true,
            "Paused": false,
            "Restarting": false,
            "OOMKilled": false,
            "Dead": false,
            "Pid": 27625,
            "ConmonPid": 27612,
            "ExitCode": 0,
            "Error": "",
            "StartedAt": "2022-10-21T23:47:24.95366414-04:00",
            "FinishedAt": "0001-01-01T00:00:00Z",
            "Healthcheck": {
                "Status": "",
                "FailingStreak": 0,
                "Log": null
            }
        },
        "Image": "538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b",
        "ImageName": "registry.access.redhat.com/rhel:latest",
        "Rootfs": "",
        "Pod": "",
        "ResolvConfPath": "/var/run/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/resolv.conf",
        "HostnamePath": "/var/run/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/hostname",
        "HostsPath": "/var/run/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/hosts",
        "StaticDir": "/var/lib/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata",
        "OCIConfigPath": "/var/lib/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/config.json",
        "OCIRuntime": "runc",
        "ConmonPidFile": "/var/run/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/conmon.pid",
        "PidFile": "/var/run/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/pidfile",
        "Name": "eager_bell",
        "RestartCount": 0,
        "Driver": "overlay",
        "MountLabel": "system_u:object_r:container_file_t:s0:c163,c861",
        "ProcessLabel": "system_u:system_r:container_t:s0:c163,c861",
        "AppArmorProfile": "",
        "EffectiveCaps": [
            "CAP_CHOWN",
            "CAP_DAC_OVERRIDE",
            "CAP_FOWNER",
            "CAP_FSETID",
            "CAP_KILL",
            "CAP_NET_BIND_SERVICE",
            "CAP_NET_RAW",
            "CAP_SETFCAP",
            "CAP_SETGID",
            "CAP_SETPCAP",
            "CAP_SETUID",
            "CAP_SYS_CHROOT"
        ],
        "BoundingCaps": [
            "CAP_CHOWN",
            "CAP_DAC_OVERRIDE",
            "CAP_FOWNER",
            "CAP_FSETID",
            "CAP_KILL",
            "CAP_NET_BIND_SERVICE",
            "CAP_NET_RAW",
            "CAP_SETFCAP",
            "CAP_SETGID",
            "CAP_SETPCAP",
            "CAP_SETUID",
            "CAP_SYS_CHROOT"
        ],
        "ExecIDs": [],
        "GraphDriver": {
            "Name": "overlay",
            "Data": {
                "LowerDir": "/var/lib/containers/storage/overlay/37a6f5f155300a48480d92a4ecf01c8694e39c3f8a52f77a39f154e5e0a3598f/diff:/var/lib/containers/storage/overlay/a27a6e18a918e68cdc3db82956cd2c0bba42d34c1513f1a0ab841903762d72a2/diff",
                "MergedDir": "/var/lib/containers/storage/overlay/de84239ee747de645c453b3b81e10849ea3e957e4316bc84ed3d0c32d7de88ee/merged",
                "UpperDir": "/var/lib/containers/storage/overlay/de84239ee747de645c453b3b81e10849ea3e957e4316bc84ed3d0c32d7de88ee/diff",
                "WorkDir": "/var/lib/containers/storage/overlay/de84239ee747de645c453b3b81e10849ea3e957e4316bc84ed3d0c32d7de88ee/work"
            }
        },
        "Mounts": [],
        "Dependencies": [],
        "NetworkSettings": {
            "EndpointID": "",
            "Gateway": "10.88.0.1",
            "IPAddress": "10.88.0.9",
            "IPPrefixLen": 16,
            "IPv6Gateway": "",
            "GlobalIPv6Address": "",
            "GlobalIPv6PrefixLen": 0,
            "MacAddress": "d6:1b:73:04:81:03",
            "Bridge": "",
            "SandboxID": "",
            "HairpinMode": false,
            "LinkLocalIPv6Address": "",
            "LinkLocalIPv6PrefixLen": 0,
            "Ports": {},
            "SandboxKey": "/run/netns/cni-f925b703-b150-a170-20e9-b52030a89784",
            "Networks": {
                "podman": {
                    "EndpointID": "",
                    "Gateway": "10.88.0.1",
                    "IPAddress": "10.88.0.9",
                    "IPPrefixLen": 16,
                    "IPv6Gateway": "",
                    "GlobalIPv6Address": "",
                    "GlobalIPv6PrefixLen": 0,
                    "MacAddress": "d6:1b:73:04:81:03",
                    "NetworkID": "podman",
                    "DriverOpts": null,
                    "IPAMConfig": null,
                    "Links": null
                }
            }
        },
        "ExitCommand": [
            "/usr/bin/podman",
            "--root",
            "/var/lib/containers/storage",
            "--runroot",
            "/var/run/containers/storage",
            "--log-level",
            "warning",
            "--cgroup-manager",
            "systemd",
            "--tmpdir",
            "/var/run/libpod",
            "--runtime",
            "runc",
            "--storage-driver",
            "overlay",
            "--storage-opt",
            "overlay.mountopt=nodev,metacopy=on",
            "--events-backend",
            "file",
            "container",
            "cleanup",
            "aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8"
        ],
        "Namespace": "",
        "IsInfra": false,
        "Config": {
            "Hostname": "aeaea3ead527",
            "Domainname": "",
            "User": "",
            "AttachStdin": false,
            "AttachStdout": false,
            "AttachStderr": false,
            "Tty": false,
            "OpenStdin": false,
            "StdinOnce": false,
            "Env": [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "TERM=xterm",
                "container=oci",
                "HOME=/root",
                "HOSTNAME=aeaea3ead527"
            ],
            "Cmd": [
                "sleep",
                "1000000"
            ],
            "Image": "registry.access.redhat.com/rhel:latest",
            "Volumes": null,
            "WorkingDir": "/",
            "Entrypoint": "",
            "OnBuild": null,
            "Labels": {
                "architecture": "x86_64",
                "build-date": "2021-07-13T15:12:27.073772",
                "com.redhat.build-host": "cpt-1006.osbs.prod.upshift.rdu2.redhat.com",
                "com.redhat.component": "rhel-server-container",
                "com.redhat.license_terms": "https://www.redhat.com/agreements",
                "description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "distribution-scope": "public",
                "io.k8s.description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "io.k8s.display-name": "Red Hat Enterprise Linux 7",
                "io.openshift.tags": "base rhel7",
                "name": "rhel7",
                "release": "437",
                "summary": "Provides the latest release of Red Hat Enterprise Linux 7 in a fully featured and supported base image.",
                "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhel7/images/7.9-437",
                "vcs-ref": "a4d1f0b8a9b923ca309da182943d17fe639d8c95",
                "vcs-type": "git",
                "vendor": "Red Hat, Inc.",
                "version": "7.9"
            },
            "Annotations": {
                "io.container.manager": "libpod",
                "io.kubernetes.cri-o.Created": "2022-10-21T23:47:24.506159696-04:00",
                "io.kubernetes.cri-o.TTY": "false",
                "io.podman.annotations.autoremove": "FALSE",
                "io.podman.annotations.init": "FALSE",
                "io.podman.annotations.privileged": "FALSE",
                "io.podman.annotations.publish-all": "FALSE",
                "org.opencontainers.image.stopSignal": "15"
            },
            "StopSignal": 15,
            "CreateCommand": [
                "podman",
                "run",
                "538460c14d75",
                "sleep",
                "1000000"
            ],
            "Umask": "0022",
            "Timeout": 0,
            "StopTimeout": 10
        },
        "HostConfig": {
            "Binds": [],
            "CgroupManager": "systemd",
            "CgroupMode": "host",
            "ContainerIDFile": "",
            "LogConfig": {
                "Type": "k8s-file",
                "Config": null,
                "Path": "/var/lib/containers/storage/overlay-containers/aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8/userdata/ctr.log",
                "Tag": "",
                "Size": "0B"
            },
            "NetworkMode": "bridge",
            "PortBindings": {},
            "RestartPolicy": {
                "Name": "",
                "MaximumRetryCount": 0
            },
            "AutoRemove": false,
            "VolumeDriver": "",
            "VolumesFrom": null,
            "CapAdd": [],
            "CapDrop": [
                "CAP_AUDIT_WRITE",
                "CAP_MKNOD"
            ],
            "Dns": [],
            "DnsOptions": [],
            "DnsSearch": [],
            "ExtraHosts": [],
            "GroupAdd": [],
            "IpcMode": "private",
            "Cgroup": "",
            "Cgroups": "default",
            "Links": null,
            "OomScoreAdj": 0,
            "PidMode": "private",
            "Privileged": false,
            "PublishAllPorts": false,
            "ReadonlyRootfs": false,
            "SecurityOpt": [],
            "Tmpfs": {},
            "UTSMode": "private",
            "UsernsMode": "",
            "ShmSize": 65536000,
            "Runtime": "oci",
            "ConsoleSize": [
                0,
                0
            ],
            "Isolation": "",
            "CpuShares": 0,
            "Memory": 0,
            "NanoCpus": 0,
            "CgroupParent": "",
            "BlkioWeight": 0,
            "BlkioWeightDevice": null,
            "BlkioDeviceReadBps": null,
            "BlkioDeviceWriteBps": null,
            "BlkioDeviceReadIOps": null,
            "BlkioDeviceWriteIOps": null,
            "CpuPeriod": 0,
            "CpuQuota": 0,
            "CpuRealtimePeriod": 0,
            "CpuRealtimeRuntime": 0,
            "CpusetCpus": "",
            "CpusetMems": "",
            "Devices": [],
            "DiskQuota": 0,
            "KernelMemory": 0,
            "MemoryReservation": 0,
            "MemorySwap": 0,
            "MemorySwappiness": 0,
            "OomKillDisable": false,
            "PidsLimit": 2048,
            "Ulimits": [
                {
                    "Name": "RLIMIT_NOFILE",
                    "Soft": 1048576,
                    "Hard": 1048576
                },
                {
                    "Name": "RLIMIT_NPROC",
                    "Soft": 32768,
                    "Hard": 32768
                }
            ],
            "CpuCount": 0,
            "CpuPercent": 0,
            "IOMaximumIOps": 0,
            "IOMaximumBandwidth": 0,
            "CgroupConf": null
        }
    }
]
""".strip()

INSPECT_2 = """
[
    {
        "Id": "28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e",
        "Created": "2022-10-21T23:49:20.02296977-04:00",
        "Path": "sleep",
        "Args": [
            "1000000"
        ],
        "State": {
            "OciVersion": "1.0.2-dev",
            "Status": "running",
            "Running": true,
            "Paused": false,
            "Restarting": false,
            "OOMKilled": false,
            "Dead": false,
            "Pid": 27942,
            "ConmonPid": 27921,
            "ExitCode": 0,
            "Error": "",
            "StartedAt": "2022-10-21T23:49:20.455664509-04:00",
            "FinishedAt": "0001-01-01T00:00:00Z",
            "Healthcheck": {
                "Status": "",
                "FailingStreak": 0,
                "Log": null
            }
        },
        "Image": "538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b",
        "ImageName": "registry.access.redhat.com/rhel:latest",
        "Rootfs": "",
        "Pod": "",
        "ResolvConfPath": "/var/run/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/resolv.conf",
        "HostnamePath": "/var/run/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/hostname",
        "HostsPath": "/var/run/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/hosts",
        "StaticDir": "/var/lib/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata",
        "OCIConfigPath": "/var/lib/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/config.json",
        "OCIRuntime": "runc",
        "ConmonPidFile": "/var/run/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/conmon.pid",
        "PidFile": "/var/run/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/pidfile",
        "Name": "youthful_goodall",
        "RestartCount": 0,
        "Driver": "overlay",
        "MountLabel": "system_u:object_r:container_file_t:s0:c1022,c1023",
        "ProcessLabel": "",
        "AppArmorProfile": "",
        "EffectiveCaps": [
            "CAP_AUDIT_CONTROL",
            "CAP_AUDIT_READ",
            "CAP_AUDIT_WRITE",
            "CAP_BLOCK_SUSPEND",
            "CAP_CHOWN",
            "CAP_DAC_OVERRIDE",
            "CAP_DAC_READ_SEARCH",
            "CAP_FOWNER",
            "CAP_FSETID",
            "CAP_IPC_LOCK",
            "CAP_IPC_OWNER",
            "CAP_KILL",
            "CAP_LEASE",
            "CAP_LINUX_IMMUTABLE",
            "CAP_MAC_ADMIN",
            "CAP_MAC_OVERRIDE",
            "CAP_MKNOD",
            "CAP_NET_ADMIN",
            "CAP_NET_BIND_SERVICE",
            "CAP_NET_BROADCAST",
            "CAP_NET_RAW",
            "CAP_SETFCAP",
            "CAP_SETGID",
            "CAP_SETPCAP",
            "CAP_SETUID",
            "CAP_SYSLOG",
            "CAP_SYS_ADMIN",
            "CAP_SYS_BOOT",
            "CAP_SYS_CHROOT",
            "CAP_SYS_MODULE",
            "CAP_SYS_NICE",
            "CAP_SYS_PACCT",
            "CAP_SYS_PTRACE",
            "CAP_SYS_RAWIO",
            "CAP_SYS_RESOURCE",
            "CAP_SYS_TIME",
            "CAP_SYS_TTY_CONFIG",
            "CAP_WAKE_ALARM"
        ],
        "BoundingCaps": [
            "CAP_AUDIT_CONTROL",
            "CAP_AUDIT_READ",
            "CAP_AUDIT_WRITE",
            "CAP_BLOCK_SUSPEND",
            "CAP_CHOWN",
            "CAP_DAC_OVERRIDE",
            "CAP_DAC_READ_SEARCH",
            "CAP_FOWNER",
            "CAP_FSETID",
            "CAP_IPC_LOCK",
            "CAP_IPC_OWNER",
            "CAP_KILL",
            "CAP_LEASE",
            "CAP_LINUX_IMMUTABLE",
            "CAP_MAC_ADMIN",
            "CAP_MAC_OVERRIDE",
            "CAP_MKNOD",
            "CAP_NET_ADMIN",
            "CAP_NET_BIND_SERVICE",
            "CAP_NET_BROADCAST",
            "CAP_NET_RAW",
            "CAP_SETFCAP",
            "CAP_SETGID",
            "CAP_SETPCAP",
            "CAP_SETUID",
            "CAP_SYSLOG",
            "CAP_SYS_ADMIN",
            "CAP_SYS_BOOT",
            "CAP_SYS_CHROOT",
            "CAP_SYS_MODULE",
            "CAP_SYS_NICE",
            "CAP_SYS_PACCT",
            "CAP_SYS_PTRACE",
            "CAP_SYS_RAWIO",
            "CAP_SYS_RESOURCE",
            "CAP_SYS_TIME",
            "CAP_SYS_TTY_CONFIG",
            "CAP_WAKE_ALARM"
        ],
        "ExecIDs": [],
        "GraphDriver": {
            "Name": "overlay",
            "Data": {
                "LowerDir": "/var/lib/containers/storage/overlay/37a6f5f155300a48480d92a4ecf01c8694e39c3f8a52f77a39f154e5e0a3598f/diff:/var/lib/containers/storage/overlay/a27a6e18a918e68cdc3db82956cd2c0bba42d34c1513f1a0ab841903762d72a2/diff",
                "MergedDir": "/var/lib/containers/storage/overlay/4be93ab17659c0d187211cd4ab4a49463b3d74f77aebc9e0c958479f9a0a3304/merged",
                "UpperDir": "/var/lib/containers/storage/overlay/4be93ab17659c0d187211cd4ab4a49463b3d74f77aebc9e0c958479f9a0a3304/diff",
                "WorkDir": "/var/lib/containers/storage/overlay/4be93ab17659c0d187211cd4ab4a49463b3d74f77aebc9e0c958479f9a0a3304/work"
            }
        },
        "Mounts": [],
        "Dependencies": [],
        "NetworkSettings": {
            "EndpointID": "",
            "Gateway": "10.88.0.1",
            "IPAddress": "10.88.0.10",
            "IPPrefixLen": 16,
            "IPv6Gateway": "",
            "GlobalIPv6Address": "",
            "GlobalIPv6PrefixLen": 0,
            "MacAddress": "5a:a0:3f:b3:1a:98",
            "Bridge": "",
            "SandboxID": "",
            "HairpinMode": false,
            "LinkLocalIPv6Address": "",
            "LinkLocalIPv6PrefixLen": 0,
            "Ports": {},
            "SandboxKey": "/run/netns/cni-d3dae08c-c0be-7a7d-c53c-c76f720c964c",
            "Networks": {
                "podman": {
                    "EndpointID": "",
                    "Gateway": "10.88.0.1",
                    "IPAddress": "10.88.0.10",
                    "IPPrefixLen": 16,
                    "IPv6Gateway": "",
                    "GlobalIPv6Address": "",
                    "GlobalIPv6PrefixLen": 0,
                    "MacAddress": "5a:a0:3f:b3:1a:98",
                    "NetworkID": "podman",
                    "DriverOpts": null,
                    "IPAMConfig": null,
                    "Links": null
                }
            }
        },
        "ExitCommand": [
            "/usr/bin/podman",
            "--root",
            "/var/lib/containers/storage",
            "--runroot",
            "/var/run/containers/storage",
            "--log-level",
            "warning",
            "--cgroup-manager",
            "systemd",
            "--tmpdir",
            "/var/run/libpod",
            "--runtime",
            "runc",
            "--storage-driver",
            "overlay",
            "--storage-opt",
            "overlay.mountopt=nodev,metacopy=on",
            "--events-backend",
            "file",
            "container",
            "cleanup",
            "28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e"
        ],
        "Namespace": "",
        "IsInfra": false,
        "Config": {
            "Hostname": "28fb57be8bb2",
            "Domainname": "",
            "User": "",
            "AttachStdin": false,
            "AttachStdout": false,
            "AttachStderr": false,
            "Tty": false,
            "OpenStdin": false,
            "StdinOnce": false,
            "Env": [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "TERM=xterm",
                "container=oci",
                "HOME=/root",
                "HOSTNAME=28fb57be8bb2"
            ],
            "Cmd": [
                "sleep",
                "1000000"
            ],
            "Image": "registry.access.redhat.com/rhel:latest",
            "Volumes": null,
            "WorkingDir": "/",
            "Entrypoint": "",
            "OnBuild": null,
            "Labels": {
                "architecture": "x86_64",
                "build-date": "2021-07-13T15:12:27.073772",
                "com.redhat.build-host": "cpt-1006.osbs.prod.upshift.rdu2.redhat.com",
                "com.redhat.component": "rhel-server-container",
                "com.redhat.license_terms": "https://www.redhat.com/agreements",
                "description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "distribution-scope": "public",
                "io.k8s.description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "io.k8s.display-name": "Red Hat Enterprise Linux 7",
                "io.openshift.tags": "base rhel7",
                "name": "rhel7",
                "release": "437",
                "summary": "Provides the latest release of Red Hat Enterprise Linux 7 in a fully featured and supported base image.",
                "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhel7/images/7.9-437",
                "vcs-ref": "a4d1f0b8a9b923ca309da182943d17fe639d8c95",
                "vcs-type": "git",
                "vendor": "Red Hat, Inc.",
                "version": "7.9"
            },
            "Annotations": {
                "io.container.manager": "libpod",
                "io.kubernetes.cri-o.Created": "2022-10-21T23:49:20.02296977-04:00",
                "io.kubernetes.cri-o.TTY": "false",
                "io.podman.annotations.autoremove": "FALSE",
                "io.podman.annotations.init": "FALSE",
                "io.podman.annotations.privileged": "TRUE",
                "io.podman.annotations.publish-all": "FALSE",
                "org.opencontainers.image.stopSignal": "15"
            },
            "StopSignal": 15,
            "CreateCommand": [
                "podman",
                "run",
                "--privileged",
                "538460c14d75",
                "sleep",
                "1000000"
            ],
            "Umask": "0022",
            "Timeout": 0,
            "StopTimeout": 10
        },
        "HostConfig": {
            "Binds": [],
            "CgroupManager": "systemd",
            "CgroupMode": "host",
            "ContainerIDFile": "",
            "LogConfig": {
                "Type": "k8s-file",
                "Config": null,
                "Path": "/var/lib/containers/storage/overlay-containers/28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e/userdata/ctr.log",
                "Tag": "",
                "Size": "0B"
            },
            "NetworkMode": "bridge",
            "PortBindings": {},
            "RestartPolicy": {
                "Name": "",
                "MaximumRetryCount": 0
            },
            "AutoRemove": false,
            "VolumeDriver": "",
            "VolumesFrom": null,
            "CapAdd": [],
            "CapDrop": [
                "CAP_BPF",
                "CAP_CHECKPOINT_RESTORE",
                "CAP_PERFMON"
            ],
            "Dns": [],
            "DnsOptions": [],
            "DnsSearch": [],
            "ExtraHosts": [],
            "GroupAdd": [],
            "IpcMode": "private",
            "Cgroup": "",
            "Cgroups": "default",
            "Links": null,
            "OomScoreAdj": 0,
            "PidMode": "private",
            "Privileged": true,
            "PublishAllPorts": false,
            "ReadonlyRootfs": false,
            "SecurityOpt": [],
            "Tmpfs": {},
            "UTSMode": "private",
            "UsernsMode": "",
            "ShmSize": 65536000,
            "Runtime": "oci",
            "ConsoleSize": [
                0,
                0
            ],
            "Isolation": "",
            "CpuShares": 0,
            "Memory": 0,
            "NanoCpus": 0,
            "CgroupParent": "",
            "BlkioWeight": 0,
            "BlkioWeightDevice": null,
            "BlkioDeviceReadBps": null,
            "BlkioDeviceWriteBps": null,
            "BlkioDeviceReadIOps": null,
            "BlkioDeviceWriteIOps": null,
            "CpuPeriod": 0,
            "CpuQuota": 0,
            "CpuRealtimePeriod": 0,
            "CpuRealtimeRuntime": 0,
            "CpusetCpus": "",
            "CpusetMems": "",
            "Devices": [],
            "DiskQuota": 0,
            "KernelMemory": 0,
            "MemoryReservation": 0,
            "MemorySwap": 0,
            "MemorySwappiness": 0,
            "OomKillDisable": false,
            "PidsLimit": 2048,
            "Ulimits": [
                {
                    "Name": "RLIMIT_NOFILE",
                    "Soft": 1048576,
                    "Hard": 1048576
                },
                {
                    "Name": "RLIMIT_NPROC",
                    "Soft": 32768,
                    "Hard": 32768
                }
            ],
            "CpuCount": 0,
            "CpuPercent": 0,
            "IOMaximumIOps": 0,
            "IOMaximumBandwidth": 0,
            "CgroupConf": null
        }
    }
]
""".strip()

INSPECT_3 = """
[
    {
        "Id": "c7efee959ea8910d68eaa5038d3ebf62ae593bfe96757b456c06f16281394921",
        "Created": "2022-10-27T02:53:02.989997608Z",
        "Path": "sleep",
        "Args": [
            "1000000"
        ],
        "State": {
            "Status": "running",
            "Running": true,
            "Paused": false,
            "Restarting": false,
            "OOMKilled": false,
            "Dead": false,
            "Pid": 32688,
            "ExitCode": 0,
            "Error": "",
            "StartedAt": "2022-10-27T02:53:03.380640461Z",
            "FinishedAt": "0001-01-01T00:00:00Z"
        },
        "Image": "sha256:acf3e09a39c95d354539b6591298be0b0814f5d74e95e722863241192b9a079b",
        "ResolvConfPath": "/var/lib/docker/containers/c7efee959ea8910d68eaa5038d3ebf62ae593bfe96757b456c06f16281394921/resolv.conf",
        "HostnamePath": "/var/lib/docker/containers/c7efee959ea8910d68eaa5038d3ebf62ae593bfe96757b456c06f16281394921/hostname",
        "HostsPath": "/var/lib/docker/containers/c7efee959ea8910d68eaa5038d3ebf62ae593bfe96757b456c06f16281394921/hosts",
        "LogPath": "",
        "Name": "/quizzical_yalow",
        "RestartCount": 0,
        "Driver": "overlay2",
        "MountLabel": "",
        "ProcessLabel": "",
        "AppArmorProfile": "",
        "ExecIDs": null,
        "HostConfig": {
            "Binds": null,
            "ContainerIDFile": "",
            "LogConfig": {
                "Type": "journald",
                "Config": {}
            },
            "NetworkMode": "default",
            "PortBindings": {},
            "RestartPolicy": {
                "Name": "no",
                "MaximumRetryCount": 0
            },
            "AutoRemove": false,
            "VolumeDriver": "",
            "VolumesFrom": null,
            "CapAdd": null,
            "CapDrop": null,
            "Dns": [],
            "DnsOptions": [],
            "DnsSearch": [],
            "ExtraHosts": null,
            "GroupAdd": null,
            "IpcMode": "",
            "Cgroup": "",
            "Links": null,
            "OomScoreAdj": 0,
            "PidMode": "",
            "Privileged": true,
            "PublishAllPorts": false,
            "ReadonlyRootfs": false,
            "SecurityOpt": [
                "label=disable"
            ],
            "UTSMode": "",
            "UsernsMode": "",
            "ShmSize": 67108864,
            "Runtime": "docker-runc",
            "ConsoleSize": [
                0,
                0
            ],
            "Isolation": "",
            "CpuShares": 0,
            "Memory": 0,
            "NanoCpus": 0,
            "CgroupParent": "",
            "BlkioWeight": 0,
            "BlkioWeightDevice": null,
            "BlkioDeviceReadBps": null,
            "BlkioDeviceWriteBps": null,
            "BlkioDeviceReadIOps": null,
            "BlkioDeviceWriteIOps": null,
            "CpuPeriod": 0,
            "CpuQuota": 0,
            "CpuRealtimePeriod": 0,
            "CpuRealtimeRuntime": 0,
            "CpusetCpus": "",
            "CpusetMems": "",
            "Devices": [],
            "DiskQuota": 0,
            "KernelMemory": 0,
            "MemoryReservation": 0,
            "MemorySwap": 0,
            "MemorySwappiness": -1,
            "OomKillDisable": false,
            "PidsLimit": 0,
            "Ulimits": null,
            "CpuCount": 0,
            "CpuPercent": 0,
            "IOMaximumIOps": 0,
            "IOMaximumBandwidth": 0
        },
        "GraphDriver": {
            "Name": "overlay2",
            "Data": {
                "LowerDir": "/var/lib/docker/overlay2/07b7d0c623c8ffe8951295328e3532f20f08e00922b77c7532c8e7afe0f05dc9-init/diff:/var/lib/docker/overlay2/64c2a13b1b7f27b198c9bb72cd61d766803d2d110646b536068e0021927f0682/diff:/var/lib/docker/overlay2/86b61c65ca8a7f18a48f15ee80e9d032adaee38835a0b0aceca8d02d50d78dd6/diff",
                "MergedDir": "/var/lib/docker/overlay2/07b7d0c623c8ffe8951295328e3532f20f08e00922b77c7532c8e7afe0f05dc9/merged",
                "UpperDir": "/var/lib/docker/overlay2/07b7d0c623c8ffe8951295328e3532f20f08e00922b77c7532c8e7afe0f05dc9/diff",
                "WorkDir": "/var/lib/docker/overlay2/07b7d0c623c8ffe8951295328e3532f20f08e00922b77c7532c8e7afe0f05dc9/work"
            }
        },
        "Mounts": [],
        "Config": {
            "Hostname": "c7efee959ea8",
            "Domainname": "",
            "User": "",
            "AttachStdin": false,
            "AttachStdout": true,
            "AttachStderr": true,
            "Tty": false,
            "OpenStdin": false,
            "StdinOnce": false,
            "Env": [
                "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
                "container=oci"
            ],
            "Cmd": [
                "sleep",
                "1000000"
            ],
            "Image": "acf3e09a39c9",
            "Volumes": null,
            "WorkingDir": "",
            "Entrypoint": null,
            "OnBuild": null,
            "Labels": {
                "architecture": "x86_64",
                "build-date": "2022-05-12T23:59:24.895710",
                "com.redhat.build-host": "cpt-1003.osbs.prod.upshift.rdu2.redhat.com",
                "com.redhat.component": "rhel-server-container",
                "com.redhat.license_terms": "https://www.redhat.com/agreements",
                "description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "distribution-scope": "public",
                "io.k8s.description": "The Red Hat Enterprise Linux Base image is designed to be a fully supported foundation for your containerized applications. This base image provides your operations and application teams with the packages, language runtimes and tools necessary to run, maintain, and troubleshoot all of your applications. This image is maintained by Red Hat and updated regularly. It is designed and engineered to be the base layer for all of your containerized applications, middleware and utilities. When used as the source for all of your containers, only one copy will ever be downloaded and cached in your production environment. Use this image just like you would a regular Red Hat Enterprise Linux distribution. Tools like yum, gzip, and bash are provided by default. For further information on how this image was built look at the /root/anacanda-ks.cfg file.",
                "io.k8s.display-name": "Red Hat Enterprise Linux 7",
                "io.openshift.tags": "base rhel7",
                "name": "rhel7",
                "release": "702",
                "summary": "Provides the latest release of Red Hat Enterprise Linux 7 in a fully featured and supported base image.",
                "url": "https://access.redhat.com/containers/#/registry.access.redhat.com/rhel7/images/7.9-702",
                "vcs-ref": "a4d1f0b8a9b923ca309da182943d17fe639d8c95",
                "vcs-type": "git",
                "vendor": "Red Hat, Inc.",
                "version": "7.9"
            }
        },
        "NetworkSettings": {
            "Bridge": "",
            "SandboxID": "0a9908c68f05eac23b6ed79dce9843b6dda2469e1021671d58d882938bffc7b6",
            "HairpinMode": false,
            "LinkLocalIPv6Address": "",
            "LinkLocalIPv6PrefixLen": 0,
            "Ports": {},
            "SandboxKey": "/var/run/docker/netns/0a9908c68f05",
            "SecondaryIPAddresses": null,
            "SecondaryIPv6Addresses": null,
            "EndpointID": "49f752615a815c97996215e72378d7e049036eef44149e96ae26e9049641d4c1",
            "Gateway": "172.17.0.1",
            "GlobalIPv6Address": "",
            "GlobalIPv6PrefixLen": 0,
            "IPAddress": "172.17.0.3",
            "IPPrefixLen": 16,
            "IPv6Gateway": "",
            "MacAddress": "02:42:ac:11:00:03",
            "Networks": {
                "bridge": {
                    "IPAMConfig": null,
                    "Links": null,
                    "Aliases": null,
                    "NetworkID": "3c2f5c9a9a9c21994ddaee6252842b8a9fb45c24d6dabf00ee36ffb12b70f0b7",
                    "EndpointID": "49f752615a815c97996215e72378d7e049036eef44149e96ae26e9049641d4c1",
                    "Gateway": "172.17.0.1",
                    "IPAddress": "172.17.0.3",
                    "IPPrefixLen": 16,
                    "IPv6Gateway": "",
                    "GlobalIPv6Address": "",
                    "GlobalIPv6PrefixLen": 0,
                    "MacAddress": "02:42:ac:11:00:03"
                }
            }
        }
    }
]
""".strip()

INSPECT_4 = """
[
    {
        "Id": "aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8"
    }
]
""".strip()

INSPECT_5 = """
Error: error inspecting object: no such object: "testnoid"
""".strip()

RELATIVE_PATH = 'insights_containers/containers_inspect'

CONTAINERS_ID_EXPECTED_RESULT = [
    ('podman', 'aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8'),
    ('podman', '28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e'),
    ('podman', '528890e93bf71736e00a87c7a1fa33e5bb03a9a196e5b10faaa9e545e749aa54'),
    ('docker', '38fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e'),
    ('docker', '538890e93bf71736e00a87c7a1fa33e5bb03a9a196e5b10faaa9e545e749aa54')
]

EXPECTED_RESULT = [{'Id': 'aeaea3ead527', 'engine': 'podman', 'Image': '538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b', 'Config|Cmd': ["sleep", "1000000"], 'HostConfig|Privileged': False}, {'Id': '28fb57be8bb2', 'engine': 'podman', 'Image': '538460c14d75dee1504e56ad8ddb7fe039093b1530ef8f90442a454b9aa3dc8b', 'Config|Cmd': ["sleep", "1000000"], 'HostConfig|Privileged': True}, {'Id': 'c7efee959ea8', 'engine': 'docker', 'Image': 'acf3e09a39c95d354539b6591298be0b0814f5d74e95e722863241192b9a079b', 'Config|Cmd': ["sleep", "1000000"], 'HostConfig|Privileged': True}]

EXPECTED_RESULT_NG = [{'Id': '28fb57be8bb2', 'engine': 'podman'}]


def setup_function(func):
    if Specs.container_inspect_keys in filters._CACHE:
        del filters._CACHE[Specs.container_inspect_keys]
    if Specs.container_inspect_keys in filters.FILTERS:
        del filters.FILTERS[Specs.container_inspect_keys]

    if func is test_containers_inspect_datasource or func is test_containers_inspect_datasource_NG_output_1 or func is test_containers_inspect_datasource_NG_output_2:
        filters.add_filter(Specs.container_inspect_keys, ["HostConfig|Privileged", "NoSuchKey|Privileged", "Config|Cmd", "Id", "Image"])
    if func is test_containers_inspect_datasource_no_filter:
        filters.add_filter(Specs.container_inspect_keys, [])


def test_running_rhel_containers_id():
    broker = dr.Broker()
    containers_info = [
        ("registry.access.redhat.com/rhel", "podman", "aeaea3ead52724bb525bb2b5c619d67836250756920f0cb9884431ba53b476d8"),
        ("registry.access.redhat.com/rhel", "podman", "28fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e"),
        ("registry.access.redhat.com/rhel", "podman", "528890e93bf71736e00a87c7a1fa33e5bb03a9a196e5b10faaa9e545e749aa54"),
        ("registry.access.redhat.com/rhel", "docker", "38fb57be8bb204e652c472a406e0d99956c8d35d6e88abfc13253d101a00911e"),
        ("registry.access.redhat.com/rhel", "docker", "538890e93bf71736e00a87c7a1fa33e5bb03a9a196e5b10faaa9e545e749aa54")
    ]
    broker[running_rhel_containers] = containers_info
    result = running_rhel_containers_id(broker)
    assert result == CONTAINERS_ID_EXPECTED_RESULT


def test_containers_inspect_datasource():
    containers_inspect_data_1 = Mock()
    containers_inspect_data_2 = Mock()
    containers_inspect_data_3 = Mock()
    containers_inspect_data_1.content = INSPECT_1.splitlines()
    containers_inspect_data_1.cmd = "/usr/bin/podman inspect aeaea3ead527"
    containers_inspect_data_2.content = INSPECT_2.splitlines()
    containers_inspect_data_2.cmd = "/usr/bin/podman inspect 28fb57be8bb2"
    containers_inspect_data_3.content = INSPECT_3.splitlines()
    containers_inspect_data_3.cmd = "/usr/bin/docker inspect c7efee959ea8"
    broker = {LocalSpecs.containers_inspect_data_raw: [containers_inspect_data_1, containers_inspect_data_2, containers_inspect_data_3]}
    result = containers_inspect_data_datasource(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=json.dumps(EXPECTED_RESULT), relative_path=RELATIVE_PATH)
    assert result.content[0] == expected.content[0]
    assert result.relative_path == expected.relative_path


def test_containers_inspect_datasource_no_filter():
    containers_inspect_data_1 = Mock()
    containers_inspect_data_2 = Mock()
    containers_inspect_data_1.content = INSPECT_1.splitlines()
    containers_inspect_data_1.cmd = "/usr/bin/docker inspect aeaea3ead527"
    containers_inspect_data_2.content = INSPECT_2.splitlines()
    containers_inspect_data_2.cmd = "/usr/bin/podman inspect 28fb57be8bb2"
    broker = {LocalSpecs.containers_inspect_data_raw: [containers_inspect_data_1, containers_inspect_data_2]}
    with pytest.raises(SkipComponent) as e:
        containers_inspect_data_datasource(broker)
    assert 'SkipComponent' in str(e)


def test_containers_inspect_datasource_NG_output_1():
    containers_inspect_data_3 = Mock()
    containers_inspect_data_3.content = INSPECT_4.splitlines()
    containers_inspect_data_3.cmd = "/usr/bin/podman inspect 28fb57be8bb2"
    broker = {LocalSpecs.containers_inspect_data_raw: [containers_inspect_data_3]}
    result = containers_inspect_data_datasource(broker)
    assert result is not None
    assert isinstance(result, DatasourceProvider)
    expected = DatasourceProvider(content=json.dumps(EXPECTED_RESULT_NG), relative_path=RELATIVE_PATH)
    assert result.content[0] == expected.content[0]
    assert result.relative_path == expected.relative_path


def test_containers_inspect_datasource_NG_output_2():
    containers_inspect_data_4 = Mock()
    containers_inspect_data_4.content = INSPECT_5.splitlines()
    containers_inspect_data_4.cmd = "/usr/bin/docker inspect aeaea3ead527"
    broker = {LocalSpecs.containers_inspect_data_raw: [containers_inspect_data_4]}
    with pytest.raises(SkipComponent) as e:
        containers_inspect_data_datasource(broker)
    assert 'Unexpected content exception' in str(e)



import pytest
import doctest
from insights.core.dr import SkipComponent
from insights.parsers import ntp_sources
from insights.parsers.ntp_sources import ChronycSources, NtpqPn, NtpqLeap
from insights.tests import context_wrap

chrony_output = """
210 Number of sources = 3
MS Name/IP address Stratum Poll Reach LastRx Last sample
===============================================================================
#* GPS0            0        4    377   11  -479ns[ -621ns]  +/- 134ns
^? a.b.c 2 6 377 23 -923us[ -924us] +/- 43ms
^+ d.e.f 1 6 377 21 -2629us[-2619us] +/- 86ms
""".strip()

chrony_output_doc = """
210 Number of sources = 6
MS Name/IP address         Stratum Poll Reach LastRx Last sample
===============================================================================
^- 10.20.30.40                   2   9   377    95  -1345us[-1345us] +/-   87ms
^- 10.56.72.8                    2  10   377   949  -3449us[-3483us] +/-  120ms
^* 10.64.108.95                  2  10   377   371    -91us[ -128us] +/-   30ms
^- 10.8.205.17                   2   8   377    27  +7161us[+7161us] +/-   52ms
""".strip()

empty_chrony_source = ""

empty_ntpq_leap = ""

ntpq_leap_output = """
leap=00
""".strip()

ntpq_leap_output_2 = """
assID=0 status=06f4 leap_none, sync_ntp, 15 events, event_peer/strat_chg,
leap=00
""".strip()

ntpd_output = """
     remote           refid      st t when poll reach   delay   offset  jitter
==============================================================================
*ntp103.cm4.tbsi 10.225.208.100   2 u  225  256  377    0.464    0.149   0.019
+ntp104.cm4.tbsi 10.228.209.150   2 u  163  256  377    0.459   -0.234   0.05
""".strip()

ntpd_qn = """
     remote           refid      st t when poll reach   delay   offset  jitter
==============================================================================
 202.118.1.81    .INIT.          16 u    - 1024    0    0.000    0.000   0.000
""".strip()

ntpd_qn_doc = """
     remote           refid      st t when poll reach   delay   offset  jitter
==============================================================================
+10.20.30.40     192.231.203.132  3 u  638 1024  377    0.242    2.461   1.886
*2001:388:608c:8 .GPS.            1 u  371 1024  377   29.323    1.939   1.312
-2001:44b8:1::1  216.218.254.202  2 u  396 1024  377   37.869   -3.340   6.458
+150.203.1.10    202.6.131.118    2 u  509 1024  377   20.135    0.800   3.260
""".strip()

empty_ntpq_pn = ""

ntp_connection_issue = """
/usr/sbin/ntpq: read: Connection refused
""".strip()


def test_get_chrony_sources():
    parser_result = ChronycSources(context_wrap(chrony_output))
    assert parser_result.data[1].get("source") == "a.b.c"
    assert parser_result.data[2].get("state") == "+"
    assert parser_result.data[2].get("mode") == "^"

    with pytest.raises(SkipComponent):
        NtpqPn(context_wrap(empty_chrony_source))


def test_get_ntpq_leap():
    parser_result = NtpqLeap(context_wrap(ntpq_leap_output))
    assert parser_result.leap == "00"

    parser_result = NtpqLeap(context_wrap(ntpq_leap_output_2))
    assert parser_result.leap == "00"

    with pytest.raises(SkipComponent) as e:
        NtpqLeap(context_wrap(ntp_connection_issue))
    assert "NTP service is down" in str(e)

    with pytest.raises(SkipComponent):
        NtpqLeap(context_wrap(empty_ntpq_leap))


def test_get_ntpd_sources():
    parser_result = NtpqPn(context_wrap(ntpd_output))
    assert parser_result.data[0].get("source") == "ntp103.cm4.tbsi"
    assert parser_result.data[1].get("flag") == "+"
    assert parser_result.data[1].get("source") == "ntp104.cm4.tbsi"

    parser_result2 = NtpqPn(context_wrap(ntpd_qn))
    assert parser_result2.data[0].get("source") == "202.118.1.81"
    assert parser_result2.data[0].get("flag") == " "

    with pytest.raises(SkipComponent) as e:
        NtpqPn(context_wrap(ntp_connection_issue))
    assert "NTP service is down" in str(e)

    with pytest.raises(SkipComponent):
        NtpqPn(context_wrap(empty_ntpq_pn))


def test_ntp_sources_doc_examples():
    env = {
        'chrony_sources': ChronycSources(context_wrap(chrony_output_doc)),
        'ntpq': NtpqLeap(context_wrap(ntpq_leap_output)),
        'ntp_sources': NtpqPn(context_wrap(ntpd_qn_doc)),
    }
    failed, total = doctest.testmod(ntp_sources, globs=env)
    assert failed == 0



import doctest
import pytest

from insights.parsers import smt, SkipException
from insights.parsers.smt import CpuSMTActive, CpuSMTControl, CpuCoreOnline, CpuSiblings
from insights.tests import context_wrap


@pytest.mark.parametrize("setting, on", [
    ("0", False),
    ("1", True)
])
def test_cpu_smt_active(setting, on):
    p = CpuSMTActive(context_wrap(setting))
    assert p.on == on


@pytest.mark.parametrize("setting, on, modifiable, supported", [
    ("on", True, True, True),
    ("off", False, True, True),
    ("forceoff", False, False, True),
    ("notsupported", False, False, False)
])
def test_cpu_smt_control(setting, on, modifiable, supported):
    p = CpuSMTControl(context_wrap(setting))
    assert p.on == on
    assert p.modifiable == modifiable
    assert p.supported == supported


@pytest.mark.parametrize("setting, on, status", [
    (0, False, "Offline"),
    (1, True, "Online")
])
def test_cpu_core_online(setting, on, status):
    path = "/sys/devices/system/cpu/cpu{0}/online"

    p = CpuCoreOnline(context_wrap(str(setting), path=path.format(setting)))
    assert p.core_id == setting
    assert p.on == on
    assert repr(p) == "[Core {0}: {1}]".format(setting, status)


@pytest.mark.parametrize("setting, core_id, siblings", [
    ("0,2", 0, [0, 2]),
    ("1-3", 3, [1, 2, 3]),
    ("1", 1, [1])
])
def test_cpu_siblings(setting, core_id, siblings):
    path = "/sys/devices/system/cpu/cpu{0}/topology/thread_siblings_list"

    p = CpuSiblings(context_wrap(setting, path=path.format(core_id)))
    assert p.core_id == core_id
    assert p.siblings == siblings
    assert repr(p) == "[Core {0} Siblings: {1}]".format(core_id, siblings)


@pytest.mark.parametrize("parser", [
    CpuSMTActive,
    CpuSMTControl,
    CpuCoreOnline,
    CpuSiblings
])
def test_exceptions(parser):
    with pytest.raises(SkipException):
        parser(context_wrap(""))


def test_doc_examples():
    path_cpu_core_online = "/sys/devices/system/cpu/cpu0/online"
    path_cpu_siblings = "/sys/devices/system/cpu/cpu0/topology/thread_siblings_list"

    env = {
        "cpu_smt_active": CpuSMTActive(context_wrap("1")),
        "cpu_smt_control": CpuSMTControl(context_wrap("off")),
        "cpu_core_online": CpuCoreOnline(context_wrap("1", path=path_cpu_core_online)),
        "cpu_siblings": CpuSiblings(context_wrap("0,2", path=path_cpu_siblings)),
    }
    failed, total = doctest.testmod(smt, globs=env)
    assert failed == 0



from insights.parsers.sysconfig import KdumpSysconfig
from insights.tests import context_wrap

SYSCONFIG_KDUMP_ALL = """
# Comments
KDUMP_KERNELVER=""

KDUMP_COMMANDLINE=""
KDUMP_COMMANDLINE_REMOVE="hugepages hugepagesz slub_debug quiet"
KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never"
KEXEC_ARGS="--elf32-core-headers"
KDUMP_IMG="vmlinuz"
KDUMP_IMG_EXT=""
"""

SYSCONFIG_KDUMP_SOME = """
# Comments
# Comments with apostrophes won't fool the dequoting process
KDUMP_COMMANDLINE_APPEND="irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never"
KDUMP_IMG="vmlinuz"
"""


def test_sysconfig_kdump():
    sc_kdump = KdumpSysconfig(context_wrap(SYSCONFIG_KDUMP_ALL))
    assert sc_kdump is not None
    assert sc_kdump.KDUMP_KERNELVER == ""
    assert sc_kdump.KDUMP_COMMANDLINE == ""
    assert sc_kdump.KDUMP_COMMANDLINE_REMOVE == "hugepages hugepagesz slub_debug quiet"
    assert sc_kdump.KDUMP_COMMANDLINE_APPEND == "irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never"
    assert sc_kdump.KEXEC_ARGS == "--elf32-core-headers"
    assert sc_kdump.KDUMP_IMG == "vmlinuz"
    assert sc_kdump.KDUMP_IMG_EXT == ""
    assert sc_kdump.get("KDUMP_IMG") == "vmlinuz"

    sc_kdump = KdumpSysconfig(context_wrap(SYSCONFIG_KDUMP_SOME))
    assert sc_kdump is not None
    assert sc_kdump.KDUMP_KERNELVER == ""
    assert sc_kdump.KDUMP_COMMANDLINE == ""
    assert sc_kdump.KDUMP_COMMANDLINE_REMOVE == ""
    assert sc_kdump.KDUMP_COMMANDLINE_APPEND == "irqpoll nr_cpus=1 reset_devices cgroup_disable=memory mce=off numa=off udev.children-max=2 panic=10 rootflags=nofail acpi_no_memhotplug transparent_hugepage=never"
    assert sc_kdump.KEXEC_ARGS == ""
    assert sc_kdump.KDUMP_IMG == "vmlinuz"
    assert sc_kdump.KDUMP_IMG_EXT == ""
    assert sc_kdump.get("KDUMP_IMG") == "vmlinuz"



"""
LsPci - Commands ``lspci``
==========================

The parsers in this module are to parse the PCI device information gathered
from the ``/sbin/lspci`` commands.

LsPci - Command ``lspci -k``
----------------------------

LsPciVmmkn - Command ``lspci -vmmkn``
-------------------------------------

"""
import re

from insights import LogFileOutput, parser, CommandParser, get_active_lines
from insights.parsers import SkipException
from insights.specs import Specs


@parser(Specs.lspci)
class LsPci(CommandParser, LogFileOutput):
    """
    Class to parse the PCI device information gathered from the
    ``/sbin/lspci -k`` command.

    Typical output of the ``lspci -k`` command is::

        00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)
                Subsystem: Cisco Systems Inc Device 0101
        00:01.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 1 (rev 13)
                Kernel driver in use: pcieport
                Kernel modules: shpchp
        00:02.0 PCI bridge: Intel Corporation 5520/5500/X58 I/O Hub PCI Express Root Port 2 (rev 13)
                Kernel driver in use: pcieport
                Kernel modules: shpchp
        03:00.0 Network controller: Intel Corporation Centrino Advanced-N 6205 [Taylor Peak] (rev 34)
                Subsystem: Cisco Systems Inc Device 004a
                Kernel driver in use: ixgbe
                Kernel modules: ixgbe
        06:00.0 Ethernet controller: Intel Corporation 82598EB 10-Gigabit AF Dual Port Network Connection (rev 01)
                Subsystem: Cisco Systems Inc Device 004a
                Kernel driver in use: ixgbe
                Kernel modules: ixgbe

    Examples:
        >>> type(lspci)
        <class 'insights.parsers.lspci.LsPci'>
        >>> lspci.get("Intel Corporation")[0]['raw_message']
        '00:00.0 Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)'
        >>> len(lspci.get("Network controller"))
        1
        >>> "Centrino Advanced-N 6205" in lspci
        True
        >>> "0d:00.0" in lspci
        False
        >>> sorted(lspci.pci_dev_list)
        ['00:00.0', '00:01.0', '00:02.0', '03:00.0', '06:00.0']
        >>> lspci.pci_dev_details('00:00.0')['Slot']
        '00:00.0'
        >>> lspci.pci_dev_details('00:00.0')['Subsystem']
        'Cisco Systems Inc Device 0101'
        >>> lspci.pci_dev_details('00:00.0')['Dev_Details']
        'Host bridge: Intel Corporation 5500 I/O Hub to ESI Port (rev 13)'

    Attributes:
        data (dict): Dict where the keys are the device number and values are
            details of the device.
        lines (list): List of details of each listed device, the same to the
            values of `self.data`

    """
    def parse_content(self, content):
        # Use all the defined scanners to search the log file, setting the
        # properties defined in the scanner.
        self.lines = [l for l in content if len(l) > 0 and l[0].isdigit()]
        for scanner in self.scanners:
            scanner(self)
        # Parse kernel driver lines
        self.data = {}
        slot = None
        slot_re = re.compile(r'^[0-9a-f]+:[0-9a-f]+.[0-9a-f]+')

        fields = ["Subsystem", "Kernel driver in use", "Kernel modules"]

        for line in get_active_lines(content):
            parts = line.split()

            if slot_re.match(parts[0]):
                slot = parts[0]
                device_details = line.split(None, 1)[-1]  # keep the raw line
                self.data[slot] = {
                        'Slot': slot,
                        'Dev_Details': device_details.lstrip()}
            elif slot and (line.split(":")[0].strip() in fields):
                parts = line.split(':')
                self.data[slot][parts[0]] = parts[1].lstrip()

    def pci_dev_details(self, dev_name):
        """
        It will return the PCI device and it's details.

        Args:
            PCI Bus Device function number eg: '00:01:0'

        Returns:
            (dict): Returns device details along with 'Subsystem', 'Kernel Driver in Use', 'Kernel Modules'.
            Returns `None` if device doesn't exists
        """
        return self.data.get(dev_name, None)

    @property
    def pci_dev_list(self):
        """
        The list of PCI devices.
        """
        return list(self.data.keys())


@parser(Specs.lspci_vmmkn)
class LsPciVmmkn(CommandParser, list):
    """
    Class to parse the PCI device information gathered from the
    ``/sbin/lspci -vmmkn`` command.

    Typical output of the ``lspci -vmmkn`` command is::

        Slot:   00:00.0
        Class:  0600
        Vendor: 8086
        Device: 1237
        SVendor:    1af4
        SDevice:    1100
        Rev:    02

        Slot:   00:01.0
        Class:  0101
        Vendor: 8086
        Device: 7010
        SVendor:    1af4
        SDevice:    1100
        ProgIf: 80
        Driver: ata_piix
        Module: ata_piix
        Module: ata_generic

        Slot:   00:01.1
        Class:  0c03
        Vendor: 8086
        Device: 7020
        SVendor:    1af4
        SDevice:    1100
        Rev:    01
        Driver: uhci_hcd

        Slot:   00:03.0
        Class:  0200
        Vendor: 1af4
        Device: 1000
        SVendor:    1af4
        SDevice:    0001
        PhySlot:    3
        Driver: virtio-pci

    Examples:
        >>> type(lspci_vmmkn)
        <class 'insights.parsers.lspci.LsPciVmmkn'>
        >>> sorted(lspci_vmmkn.pci_dev_list)
        ['00:00.0', '00:01.0', '00:01.1', '00:03.0']
        >>> lspci_vmmkn[0].get('Driver') is None
        True
        >>> lspci_vmmkn[-1].get('Driver')
        'virtio-pci'
        >>> len(lspci_vmmkn[1].get('Module'))
        2

    Attributes:

    """
    def parse_content(self, content):
        # Remove the white-trailing of the output
        while content and not content[-1].strip():
            content.pop(-1)

        dev = {}
        self.append(dev)
        for line in content:
            line = line.strip()
            if not line:
                # Skip empty lines
                if dev:
                    dev = {}
                    self.append(dev)
                continue
            key, val = [i.strip() for i in line.split(':', 1)]
            # Module could have multiple values
            if key == 'Module':
                if key in dev:
                    dev[key].append(val)
                else:
                    dev[key] = [val]
            else:
                dev[key] = val

        if len(self) <= 1 and not dev:
            raise SkipException()

    @property
    def pci_dev_list(self):
        """
        The list of PCI devices.
        """
        return [i['Slot'] for i in self]



"""
Custom datasources to get a list of directories to check disk size.
"""

from insights.core.context import HostContext
from insights.core.dr import SkipComponent
from insights.core.plugins import datasource
from insights.core.filters import get_filters
from insights.specs import Specs


@datasource(HostContext)
def du_dir_list(broker):
    """ Return a list of directories from the spec filter """
    filters = list(get_filters(Specs.du_dirs))
    if filters:
        return filters
    raise SkipComponent



"""
Slab allocator's details.
=========================

SlabInfo - File ``/proc/slabinfo``
----------------------------------
"""

from insights.specs import Specs
from insights import Parser, parser
from insights.parsers import SkipException


@parser(Specs.proc_slabinfo)
class SlabInfo(Parser):
    """
    Parse the content of the ``/proc/slabinfo`` file

    Sample input data looks like::

        slabinfo - version: 2.1
        # name            <active_objs> <num_objs> <objsize> <objperslab> <pagesperslab> : tunables <limit> <batchcount> <sharedfactor> : slabdata <active_slabs> <num_slabs> <sharedavail>
        sw_flow                0      0   1256   13    4 : tunables    0    0    0 : slabdata      0      0      0
        nf_conntrack_ffffffffaf313a40     12     12    320   12    1 : tunables    0    0    0 : slabdata      1      1      0
        xfs_dqtrx              0      0    528   15    2 : tunables    0    0    0 : slabdata      0      0      0
        xfs_dquot              0      0    488    8    1 : tunables    0    0    0 : slabdata      0      0      0
        xfs_ili             2264   2736    168   24    1 : tunables    0    0    0 : slabdata    114    114      0
        xfs_inode           4845   5120    960    8    2 : tunables    0    0    0 : slabdata    640    640      0
        xfs_efd_item          76     76    416   19    2 : tunables    0    0    0 : slabdata      4      4      0
        xfs_btree_cur         18     18    216   18    1 : tunables    0    0    0 : slabdata      1      1      0
        xfs_log_ticket        22     22    184   22    1 : tunables    0    0    0 : slabdata      1      1      0
        bio-3                 60     60    320   12    1 : tunables    0    0    0 : slabdata      5      5      0
        kcopyd_job             0      0   3312    9    8 : tunables    0    0    0 : slabdata      0      0      0
        dm_uevent              0      0   2608   12    8 : tunables    0    0    0 : slabdata      0      0      0
        dm_rq_target_io        0      0    136   30    1 : tunables    0    0    0 : slabdata      0      0      0
        ip6_dst_cache         72     72    448    9    1 : tunables    0    0    0 : slabdata      8      8      0
        RAWv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
        UDPLITEv6              0      0   1216   13    4 : tunables    0    0    0 : slabdata      0      0      0
        UDPv6                 13     13   1216   13    4 : tunables    0    0    0 : slabdata      1      1      0
        tw_sock_TCPv6          0      0    256   16    1 : tunables    0    0    0 : slabdata      0      0      0
        TCPv6                 15     15   2112   15    8 : tunables    0    0    0 : slabdata      1      1      0
        cfq_queue              0      0    232   17    1 : tunables    0    0    0 : slabdata      0      0      0
        bsg_cmd                0      0    312   13    1 : tunables    10   20   30 : slabdata     40     50     60

    Examples:

        >>> type(pslabinfo)
        <class 'insights.parsers.slabinfo.SlabInfo'>
        >>> len(pslabinfo.data.keys())
        21
        >>> pslabinfo.slab_object('bsg_cmd', 'active_slabs')
        40
        >>> pslabinfo.slab_object('bsg_cmd', 'limit')
        10
    """

    def parse_content(self, content):
        self.data = {}
        self.__slab_version = None
        column = []
        row = []
        if not content:
            raise SkipException("No Contents")

        if 'slabinfo - version' in content[0]:
            self.__slab_version = content[0].split()[-1]
        else:
            raise SkipException("Invalid Contents")

        if "active_objs" in content[1]:
            line = content[1].split()
            column = [obj.replace('<', '').replace('>', '') for obj in line if obj not in ['#', ':', 'tunables', 'slabdata']]
        else:
            raise SkipException("Invalid Contents")

        for line in content[2:]:
            line = line.split()
            row = [obj for obj in line if obj not in ['#', ':', 'tunables', 'slabdata']]
            if column and row and len(column) == len(row):
                self.data[row[0]] = dict(zip(column, row))
                row = []
            else:
                raise SkipException("Invalid Contents")

    @property
    def slab_version(self):
        """
        (str): On success it will return the slab version else it will return `None`.
        """
        return self.__slab_version

    def slab_details(self, slab_name):
        """
        (dict): On success it will return the deatils of given slab, else it will return `None`.
        """
        return self.data.get(slab_name, None)

    def slab_object(self, slab_name, slab_obj):
        """
        (int): On success it will return the allocated slab object number, else it will return `0`.
        """
        slab_detail = self.data.get(slab_name, None)
        if slab_detail and slab_obj != 'name':
            return int(slab_detail.get(slab_obj, 0))



"""
JBoss standalone mode main configuration - file ``$JBOSS_BASE_DIR/standalone.xml``
==================================================================================
This parser reads the XML in the JBoss standalone mode main configuration file.

.. note::
        Please refer to its super-class :class:`insights.core.XMLParser` for more details.


Sample input::

    <?xml version='1.0' encoding='UTF-8'?>

    <server xmlns="urn:jboss:domain:1.7">
        <management>
            <security-realms>
                <security-realm name="ManagementRealm">
                    <authentication>
                        <local default-user="$local" skip-group-loading="true"/>
                        <properties path="mgmt-users.properties" relative-to="jboss.server.config.dir"/>
                    </authentication>
                    <authorization map-groups-to-roles="false">
                        <properties path="mgmt-groups.properties" relative-to="jboss.server.config.dir"/>
                    </authorization>
                </security-realm>
                <security-realm name="ApplicationRealm">
                    <authentication>
                        <local default-user="$local" allowed-users="*" skip-group-loading="true"/>
                        <properties path="application-users.properties" relative-to="jboss.server.config.dir"/>
                    </authentication>
                    <authorization>
                        <properties path="application-roles.properties" relative-to="jboss.server.config.dir"/>
                    </authorization>
                </security-realm>
            </security-realms>
        </management>
    </server>

Examples:

    >>> jboss_main_config.file_path
    '/root/jboss/jboss-eap-6.4/standalone/configuration/standalone.xml'
    >>> properties = sorted(jboss_main_config.get_elements(".//management/security-realms/security-realm/authentication/properties"), key=lambda e: e.tag)
    >>> properties[0].get("relative-to")
    'jboss.server.config.dir'

"""

from .. import XMLParser, parser

from insights.specs import Specs


@parser(Specs.jboss_standalone_main_config)
class JbossStandaloneConf(XMLParser):
    """
    Read the XML in the JBoss standalone mode main configuration file
    """

    pass



import pytest

from insights.specs import Specs
from insights.core import filters
from insights.specs.datasources.kernel_module_list import kernel_module_filters
from insights.core.dr import SkipComponent


def setup_function(func):
    if Specs.modinfo_modules in filters._CACHE:
        del filters._CACHE[Specs.modinfo_modules]
    if Specs.modinfo_modules in filters.FILTERS:
        del filters.FILTERS[Specs.modinfo_modules]

    if func is test_module_filters:
        filters.add_filter(Specs.modinfo_modules, ["udp_diag", "binfmt_misc"])
    if func is test_module_filters_empty:
        filters.add_filter(Specs.modinfo_modules, [])


def test_module_filters():
    broker = {}
    result = kernel_module_filters(broker)
    assert 'binfmt_misc udp_diag' == result


def test_module_filters_empty():
    broker = {}
    with pytest.raises(SkipComponent):
        kernel_module_filters(broker)



import doctest
import pytest
from insights.parsers import systemctl_show, SkipException, ParseException
from insights.parsers.systemctl_show import (
    SystemctlShowServiceAll, SystemctlShowTarget, SystemctlShowAllServiceWithLimitedProperties
)
from insights.tests import context_wrap
# TODO: Remove the following `import`s when removing the deprecated parsers
from insights.parsers.systemctl_show import SystemctlShowCinderVolume
from insights.parsers.systemctl_show import SystemctlShowHttpd
from insights.parsers.systemctl_show import SystemctlShowMariaDB
from insights.parsers.systemctl_show import SystemctlShowPulpCelerybeat
from insights.parsers.systemctl_show import SystemctlShowPulpResourceManager
from insights.parsers.systemctl_show import SystemctlShowPulpWorkers
from insights.parsers.systemctl_show import SystemctlShowQpidd
from insights.parsers.systemctl_show import SystemctlShowQdrouterd
from insights.parsers.systemctl_show import SystemctlShowSmartpdc
from insights.parsers.systemctl_show import SystemctlShowNginx


# TODO: Remove the following test cases when removing the deprecated parsers
SYSTEMCTL_SHOW_EXAMPLES = """
WatchdogUSec=0
WatchdogTimestamp=Thu 2018-01-11 14:22:33 CST
WatchdogTimestampMonotonic=105028136
StartLimitInterval=10000000
StartLimitBurst=5
StartLimitAction=none
FailureAction=none
PermissionsStartOnly=no
RootDirectoryStartOnly=no
RemainAfterExit=no
GuessMainPID=yes
MainPID=2810
ControlPID=0
FileDescriptorStoreMax=0
StatusErrno=0
Result=success
ExecMainStartTimestamp=Thu 2018-01-11 14:22:33 CST
ExecMainStartTimestampMonotonic=105028117
ExecMainExitTimestampMonotonic=0
ExecMainPID=2811
LimitNOFILE=4096
"""


def test_systemctl_show_cinder_volume():
    context = SystemctlShowCinderVolume(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_mariadb():
    context = SystemctlShowMariaDB(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_pulp_workers():
    context = SystemctlShowPulpWorkers(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_pulp_resource_manager():
    context = SystemctlShowPulpResourceManager(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_pulp_celerybeat():
    context = SystemctlShowPulpCelerybeat(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_httpd():
    context = SystemctlShowHttpd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_qpidd():
    context = SystemctlShowQpidd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_qdrouterd():
    context = SystemctlShowQdrouterd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_smartpdc():
    context = SystemctlShowSmartpdc(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21


def test_systemctl_show_nginx():
    context = SystemctlShowNginx(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    assert context["LimitNOFILE"] == "4096"
    assert len(context.data) == 21
# NOTE: Above code should be removed after removing the deprecated parsers


SYSTEMCTL_SHOW_ALL_EXAMPLES = """
KillSignal=15
SendSIGKILL=yes
SendSIGHUP=no
Id=postfix.service
Names=postfix.service
Requires=basic.target
Wants=system.slice
WantedBy=multi-user.target
LimitNOFILE=65536
LimitMEMLOCK=
LimitLOCKS=18446744073709551615

User=postgres
Group=postgres
MountFlags=0
PrivateTmp=no
PrivateNetwork=no
PrivateDevices=no
ProtectHome=no
ProtectSystem=no
SameProcessGroup=no
IgnoreSIGPIPE=yes
NoNewPrivileges=no
SystemCallErrorNumber=0
RuntimeDirectoryMode=0755
KillMode=control-group
KillSignal=15
SendSIGKILL=yes
SendSIGHUP=no
Id=postgresql.service
Names=postgresql.service
Requires=basic.target
LimitMSGQUEUE=819200
LimitNICE=0
ExecStartPre={ path=/usr/bin/postgresql-check-db-dir ; argv[]=/usr/bin/postgresql-check-db-dir ${PGDATA} ; ignore_errors=no ; start_time=[Tue 2019-11-19 23:55:49 EST]
ExecStart={ path=/usr/bin/pg_ctl ; argv[]=/usr/bin/pg_ctl start -D ${PGDATA} -s -o -p ${PGPORT} -w -t 300 ; ignore_errors=no ; start_time=[Tue 2019-11-19 23:55:50 EST]
ExecReload={ path=/usr/bin/pg_ctl ; argv[]=/usr/bin/pg_ctl reload -D ${PGDATA} -s ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; statu
ExecStop={ path=/usr/bin/pg_ctl ; argv[]=/usr/bin/pg_ctl stop -D ${PGDATA} -s -m fast ; ignore_errors=no ; start_time=[n/a] ; stop_time=[n/a] ; pid=0 ; code=(null) ; s
Slice=system.slice
ControlGroup=/system.slice/postgresql.service

Id=tuned.service
Names=tuned.service
Requires=polkit.service basic.target dbus.service
Wants=system.slice
WantedBy=multi-user.target
Conflicts=shutdown.target cpupower.service
Before=multi-user.target shutdown.target
After=systemd-sysctl.service system.slice network.target systemd-journald.socket
basic.target dbus.service
Documentation=man:tuned(8) man:tuned.conf(5) man:tuned-adm(8)
Description=Dynamic System Tuning Daemon
LoadState=loaded
ActiveState=active
SubState=running
FragmentPath=/usr/lib/systemd/system/tuned.service
UnitFileState=enabled
UnitFilePreset=enabled
InactiveExitTimestamp=Tue 2019-11-19 23:55:50 EST
InactiveExitTimestampMonotonic=9434667
ActiveEnterTimestamp=Tue 2019-11-19 23:55:53 EST
ActiveEnterTimestampMonotonic=12640144
ActiveExitTimestampMonotonic=0
InactiveEnterTimestampMonotonic=0
CanStart=yes
CanStop=yes
CanReload=no
CanIsolate=no
StopWhenUnneeded=no
RefuseManualStart=no
RefuseManualStop=no
AllowIsolate=no
DefaultDependencies=yes
OnFailureJobMode=replace
IgnoreOnIsolate=no
IgnoreOnSnapshot=no
NeedDaemonReload=no
JobTimeoutUSec=0
JobTimeoutAction=none
ConditionResult=yes
AssertResult=yes
ConditionTimestamp=Tue 2019-11-19 23:55:50 EST
ConditionTimestampMonotonic=9429104
AssertTimestamp=Tue 2019-11-19 23:55:50 EST
AssertTimestampMonotonic=9429105
ABC=
Transient=no
""".strip()

SYSTEMCTL_SHOW_ALL_EXP = """
KillSignal=15
SendSIGKILL=yes
SendSIGHUP=no
Requires=basic.target
Wants=system.slice
WantedBy=multi-user.target
LimitNOFILE=65536
LimitMEMLOCK=
LimitLOCKS=18446744073709551615

""".strip()

SYSTEMCTL_SHOW_TARGET = """
Id=network.target
Names=network.target
WantedBy=NetworkManager.service
Conflicts=shutdown.target
Before=tuned.service network-online.target rhsmcertd.service kdump.service httpd.service rsyslog.service rc-local.service insights-client.timer insights-client.service sshd.service postfix.service
After=firewalld.service network-pre.target network.service NetworkManager.service
Documentation=man:systemd.special(7) http://www.freedesktop.org/wiki/Software/systemd/NetworkTarget
Description=Network
LoadState=loaded
ActiveState=active
SubState=active
FragmentPath=/usr/lib/systemd/system/network.target
UnitFileState=static
UnitFilePreset=disabled
InactiveExitTimestamp=Tue 2020-02-25 10:39:46 GMT
InactiveExitTimestampMonotonic=15332468
ActiveEnterTimestamp=Tue 2020-02-25 10:39:46 GMT
ActiveEnterTimestampMonotonic=15332468
ActiveExitTimestampMonotonic=0
InactiveEnterTimestampMonotonic=0
CanStart=no

Id=swap.target
Names=swap.target
Requires=dev-mapper-rhel\x5cx2dswap.swap
WantedBy=sysinit.target
Conflicts=shutdown.target
Before=sysinit.target tmp.mount
After=dev-disk-by\x5cx2did-dm\x5cx2duuid\x5cx2dLVM\x5cx2deP73apHGoZ6LcvX230L0BUHQiPDhXceEkTtrOvL6P2biOWacMR3YS7rISVOgnPdc.swap dev-dm\x5cx2d1.swap dev-rhel-swap.swap dev-disk-by\x5cx2did-dm\x5cx2dname\x5cx2drhel\x5cx2dswap.swap dev-disk-by\x5cx2duuid-1ffaf940\x5cx2de836\x5cx2d4f08\x5cx2d9e5f\x5cx2d4dbe425d787f.swap dev-mapper-rhel\x5cx2dswap.swap
Documentation=man:systemd.special(7)
Description=Swap
LoadState=loaded
ActiveState=active
SubState=active
FragmentPath=/usr/lib/systemd/system/swap.target
UnitFileState=static
UnitFilePreset=disabled
InactiveExitTimestamp=Tue 2020-02-25 10:39:35 GMT
InactiveExitTimestampMonotonic=4692015
ActiveEnterTimestamp=Tue 2020-02-25 10:39:35 GMT
ActiveEnterTimestampMonotonic=4692015
ActiveExitTimestamp=Tue 2020-02-25 10:39:34 GMT
ActiveExitTimestampMonotonic=3432423

Id=paths.target
Names=paths.target
WantedBy=basic.target
Conflicts=shutdown.target
Before=basic.target
After=brandbot.path systemd-ask-password-console.path systemd-ask-password-wall.path
Documentation=man:systemd.special(7)
Description=Paths
LoadState=loaded
ActiveState=active
SubState=active
FragmentPath=/usr/lib/systemd/system/paths.target
UnitFileState=static
UnitFilePreset=disabled
InactiveExitTimestamp=Tue 2020-02-25 10:39:38 GMT
InactiveExitTimestampMonotonic=7782864
ActiveEnterTimestamp=Tue 2020-02-25 10:39:38 GMT
ActiveEnterTimestampMonotonic=7782864
ActiveExitTimestamp=Tue 2020-02-25 10:39:34 GMT
ActiveExitTimestampMonotonic=3427228
InactiveEnterTimestamp=Tue 2020-02-25 10:39:34 GMT

Id=sockets.target
Names=sockets.target
Wants=rpcbind.socket systemd-initctl.socket systemd-udevd-kernel.socket dbus.socket dm-event.socket systemd-udevd-control.socket systemd-journald.socket systemd-shutdownd.socket
WantedBy=basic.target
Conflicts=shutdown.target
Before=basic.target
After=systemd-udevd-kernel.socket systemd-shutdownd.socket sshd.socket systemd-udevd-control.socket systemd-initctl.socket systemd-journald.socket rpcbind.socket syslog.socket dbus.socket
Documentation=man:systemd.special(7)
Description=Sockets
LoadState=loaded
ActiveState=active
SubState=active
FragmentPath=/usr/lib/systemd/system/sockets.target
UnitFileState=static
UnitFilePreset=disabled
InactiveExitTimestamp=Tue 2020-02-25 10:39:38 GMT
InactiveExitTimestampMonotonic=7786499
ActiveEnterTimestamp=Tue 2020-02-25 10:39:38 GMT
ActiveEnterTimestampMonotonic=7786499
ActiveExitTimestamp=Tue 2020-02-25 10:39:34 GMT
ActiveExitTimestampMonotonic=3427060
""".strip()

SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO = """
CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test1.service
SubState=dead
UnitFileState=static

CPUAccounting=yes
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test2.service
SubState=dead
UnitFileState=enabled
""".strip()

SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO2 = """
CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test1.service
SubState=dead
UnitFileState=static

CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=300ms
Names=test3.service
SubState=running
UnitFileState=enabled
""".strip()

SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO3 = """
CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test1.service
SubState=dead
UnitFileState=static

CPUAccounting=no
CPUShares=184
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test2.service
SubState=failed
UnitFileState=enabled

CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=184
CPUQuotaPerSecUSec=infinity
Names=test3.service
SubState=exited
UnitFileState=enabled
""".strip()

SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO4 = """
CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test1.service
SubState=dead
UnitFileState=static

CPUAccounting=no
CPUShares=[not set]
StartupCPUShares=[not set]
CPUQuotaPerSecUSec=300ms
Names=insights-client.service
SubState=dead
UnitFileState=static
""".strip()

SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO5 = """
CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test1.service
SubState=dead
UnitFileState=static

CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test2.service
SubState=failed
UnitFileState=enabled

CPUAccounting=no
CPUShares=18446744073709551615
StartupCPUShares=18446744073709551615
CPUQuotaPerSecUSec=infinity
Names=test3.service
SubState=exited
UnitFileState=enabled
""".strip()


def test_systemctl_show_service_all():
    svc_all = SystemctlShowServiceAll(context_wrap(SYSTEMCTL_SHOW_ALL_EXAMPLES))
    assert len(svc_all) == 3

    assert 'postfix.service' in svc_all
    assert 'LimitMEMLOCK' not in svc_all['postfix.service']
    assert svc_all['postfix.service']["LimitNOFILE"] == "65536"
    assert svc_all['postfix.service']["KillSignal"] == "15"
    assert svc_all['postfix.service']["LimitLOCKS"] == "18446744073709551615"
    assert len(svc_all['postfix.service']) == 10

    assert 'postgresql.service' in svc_all
    assert svc_all['postgresql.service']["User"] == "postgres"
    assert svc_all['postgresql.service']["ControlGroup"] == "/system.slice/postgresql.service"
    assert len(svc_all['postgresql.service']) == 28

    assert 'tuned.service' in svc_all
    assert svc_all['tuned.service']["Id"] == "tuned.service"
    assert svc_all['tuned.service']["Transient"] == "no"
    assert "ABC" not in svc_all['tuned.service']
    assert len(svc_all['tuned.service']) == 44


def test_systemctl_show_target():
    data = SystemctlShowTarget(context_wrap(SYSTEMCTL_SHOW_TARGET))
    assert 'network.target' in data
    assert data.get('network.target').get('WantedBy', None) == 'NetworkManager.service'
    assert data.get('network.target').get('RequiredBy', None) is None


def test_systemctl_show_service_all_ab():
    with pytest.raises(SkipException):
        SystemctlShowServiceAll(context_wrap(''))

    with pytest.raises(ParseException):
        SystemctlShowServiceAll(context_wrap(SYSTEMCTL_SHOW_ALL_EXP))


def test_systemctl_show_service_with_cpuaccouting_enabled():
    services = SystemctlShowAllServiceWithLimitedProperties(context_wrap(SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO))
    assert 'test1.service' in services
    assert 'test3.service' not in services
    assert services.get('test1.service').get('CPUAccounting') == 'no'
    assert 'test2.service' in services.get_services_with_cpuaccouting_enabled()
    assert 'test1.service' not in services.get_services_with_cpuaccouting_enabled()

    services = SystemctlShowAllServiceWithLimitedProperties(context_wrap(SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO2))
    assert 'test3.service' in services.get_services_with_cpuaccouting_enabled()

    services = SystemctlShowAllServiceWithLimitedProperties(context_wrap(SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO3))
    assert 'test2.service' in services.get_services_with_cpuaccouting_enabled()
    assert 'test3.service' in services.get_services_with_cpuaccouting_enabled()

    services = SystemctlShowAllServiceWithLimitedProperties(context_wrap(SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO4))
    assert 'insights-client.service' in services.get_services_with_cpuaccouting_enabled()


def test_systemctl_show_doc_examples():
    env = {
        'systemctl_show_all': SystemctlShowServiceAll(context_wrap(SYSTEMCTL_SHOW_ALL_EXAMPLES)),
        'systemctl_show_target': SystemctlShowTarget(context_wrap(SYSTEMCTL_SHOW_TARGET)),
        'all_services_with_limited_info': SystemctlShowAllServiceWithLimitedProperties(context_wrap(SYSTEM_SHOW_ALL_SERVICES_WITH_CPUACCOUTING_INFO)),
        # TODO: Remove the following test when removing the deprecated parsers.
        'systemctl_show_cinder_volume': SystemctlShowCinderVolume(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_mariadb': SystemctlShowMariaDB(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_pulp_workers': SystemctlShowPulpWorkers(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_pulp_resource_manager': SystemctlShowPulpResourceManager(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_pulp_celerybeat': SystemctlShowPulpCelerybeat(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_httpd': SystemctlShowHttpd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_qpidd': SystemctlShowQpidd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_qdrouterd': SystemctlShowQdrouterd(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_smartpdc': SystemctlShowSmartpdc(context_wrap(SYSTEMCTL_SHOW_EXAMPLES)),
        'systemctl_show_nginx': SystemctlShowNginx(context_wrap(SYSTEMCTL_SHOW_EXAMPLES))
    }
    failed, total = doctest.testmod(systemctl_show, globs=env)
    assert failed == 0



import pytest
import doctest

from insights.tests import context_wrap
from insights.parsers import auditctl
from insights.parsers.auditctl import AuditStatus, AuditRules
from insights.parsers import ParseException, SkipException


NORMAL_AUDS_RHEL6 = """
AUDIT_STATUS: enabled=1 flag=1 pid=1483 rate_limit=0 backlog_limit=8192 lost=3 backlog=0
""".strip()

BAD_AUDS_RHEL6 = """
AUDIT_STATUS: enabled=1 flag=1 pid=1483 rate_limit=0 backlog_limit=8192 lost=3 backlog=0 test=test
""".strip()

NORMAL_AUDS_RHEL7 = """
enabled 1
failure 1
pid 947
rate_limit 0
backlog_limit 320
lost 0
backlog 0
loginuid_immutable 1 locked
""".strip()

BAD_AUDS_RHEL7 = """
enabled 1
failure 1
pid 947
rate_limit 0
backlog_limit 320
lost 0
backlog 0
test test
loginuid_immutable 1 locked
""".strip()

BLANK_INPUT_SAMPLE = """
""".strip()

BAD_INPUT_SAMPLE = """
Unknown: type=0, len=0
""".strip()

BAD_INPUT_MIX = """
Unknown: type=0, len=0
enabled 1
""".strip()

AUDIT_RULES_OUTPUT1 = """
No rules
""".strip()

AUDIT_RULES_OUTPUT2 = """
-w /etc/selinux -p wa -k MAC-policy
-a always,exit -F arch=b32 -S chmod -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b64 -S chmod -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b32 -S chmod -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b64 -S chmod -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access

-a always,exit -F arch=b32 -S chown -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b64 -S chown -F exit=-EACCES -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b32 -S chown -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
-a always,exit -F arch=b64 -S chown -F exit=-EPERM -F auid>=1000 -F auid!=-1 -F key=access
""".strip()

AUDIT_RULES_OUTPUT3 = """

"""


def test_normal_auds_rhel6():
    auds = AuditStatus(context_wrap(NORMAL_AUDS_RHEL6))
    assert "enabled" in auds
    assert "loginuid_immutable" not in auds
    assert auds['pid'] == 1483


def test_normal_auds_rhel7():
    auds = AuditStatus(context_wrap(NORMAL_AUDS_RHEL7))
    assert "loginuid_immutable" in auds
    assert auds['loginuid_immutable'] == "1 locked"
    assert auds['failure'] == 1
    assert auds.get('nonexists') is None


def test_auds_blank_input():
    ctx = context_wrap(BLANK_INPUT_SAMPLE)
    with pytest.raises(SkipException) as sc:
        AuditStatus(ctx)
    assert "Input content is empty." in str(sc)
    with pytest.raises(SkipException):
        AuditStatus(context_wrap(BAD_INPUT_SAMPLE))


def test_parse_exception():
    with pytest.raises(ParseException):
        AuditStatus(context_wrap(BAD_AUDS_RHEL7))
    with pytest.raises(ParseException):
        AuditStatus(context_wrap(BAD_AUDS_RHEL6))


def test_audit_rules():
    audit_rules = AuditRules(context_wrap(AUDIT_RULES_OUTPUT2))
    assert len(audit_rules) == 9
    assert '-w /etc/selinux -p wa -k MAC-policy' in audit_rules


def test_audit_rules_exception():
    with pytest.raises(SkipException):
        AuditRules(context_wrap(AUDIT_RULES_OUTPUT1))
    with pytest.raises(SkipException):
        AuditRules(context_wrap(AUDIT_RULES_OUTPUT3))


def test_doc_examples():
    env = {
        'audit_rules': AuditRules(context_wrap(AUDIT_RULES_OUTPUT2)),
        'auds': AuditStatus(context_wrap(NORMAL_AUDS_RHEL7))
    }
    failed, total = doctest.testmod(auditctl, globs=env)
    assert failed == 0



"""
JournalAll file ``/sos_commands/logs/journalctl_--no-pager``
============================================================
"""

from .. import Syslog, parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.journal_all)
class JournalAll(Syslog):
    """

    .. warning::

        This parser is deprecated, please use
        :py:class:`insights.parsers.journalctl.JournalAll` instead.


    Read the ``/sos_commands/logs/journalctl_--no-pager`` file.  Uses the
    ``Syslog`` class parser functionality - see the base class for more details.

    Sample log lines::

        -- Logs begin at Wed 2017-02-08 15:18:00 CET, end at Tue 2017-09-19 09:25:27 CEST. --
        May 18 15:13:34 lxc-rhel68-sat56 jabberd/sm[11057]: session started: jid=rhn-dispatcher-sat@lxc-rhel6-sat56.redhat.com/superclient
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon
        May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: Launching a JVM...
        May 18 15:24:28 lxc-rhel68-sat56 yum[11597]: Installed: lynx-2.8.6-27.el6.x86_64
        May 18 15:36:19 lxc-rhel68-sat56 yum[11954]: Updated: sos-3.2-40.el6.noarch

    .. note::
        Because journal timestamps by default have no year,
        the year of the logs will be inferred from the year in your timestamp.
        This will also work around December/January crossovers.

    Examples:
        >>> JournalAll.filters.append('wrapper')
        >>> JournalAll.token_scan('daemon_start', 'Wrapper Started as Daemon')
        >>> msgs = shared[JournalAll]
        >>> len(msgs.lines)
        >>> wrapper_msgs = msgs.get('wrapper') # Can only rely on lines filtered being present
        >>> wrapper_msgs[0]
        {'timestamp': 'May 18 15:13:36', 'hostname': 'lxc-rhel68-sat56',
         'procname': wrapper[11375]', 'message': '--> Wrapper Started as Daemon',
         'raw_message': 'May 18 15:13:36 lxc-rhel68-sat56 wrapper[11375]: --> Wrapper Started as Daemon'
        }
        >>> msgs.daemon_start # Token set if matching lines present in logs
        True
    """

    def __init__(self, context):
        deprecated(
            JournalAll,
            "Please use the :class:`insights.parsers.journalctl.JournalAll` instead.",
            "3.1.25"
        )
        super(JournalAll, self).__init__(context)

    pass



"""
System kernel files under ``/proc/sys/kernel`` or ``/sys/kernel``
=================================================================

This module contains the following parsers:

SchedRTRuntime - file ``/proc/sys/kernel/sched_rt_runtime_us``
--------------------------------------------------------------
SchedFeatures - file ``/sys/kernel/debug/sched_features``
---------------------------------------------------------
"""

from insights import Parser, parser, get_active_lines
from insights.parsers import ParseException
from insights.specs import Specs


@parser(Specs.sched_rt_runtime_us)
class SchedRTRuntime(Parser):
    """
    Class for parsing the `/proc/sys/kernel/sched_rt_runtime_us` file.

    Typical content of the file is::

        950000

    Examples:
        >>> type(srt)
        <class 'insights.parsers.sys_kernel.SchedRTRuntime'>
        >>> srt.runtime_us
        950000

    Attributes:
        runtime_us (int): The value of sched_rt_runtime_us

    Raises:
        ParseException: Raised when there is more than one line or the value isn't interger.
    """

    def parse_content(self, content):
        lines = get_active_lines(content)
        if len(lines) != 1:
            raise ParseException("Unexpected file content")
        try:
            self.runtime_us = int(lines[0])
        except:
            raise ParseException("Unexpected file content")


@parser(Specs.sys_kernel_sched_features)
class SchedFeatures(Parser):
    """
    Class for parsing the `/sys/kernel/debug/sched_features` file.

    Typical content of the file is::

        GENTLE_FAIR_SLEEPERS START_DEBIT NO_NEXT_BUDDY LAST_BUDDY CACHE_HOT_BUDDY

    Examples:
        >>> type(sfs)
        <class 'insights.parsers.sys_kernel.SchedFeatures'>
        >>> "GENTLE_FAIR_SLEEPERS" in sfs.features
        True
        >>> "TEST1" in sfs.features
        False

    Attributes:
        features (list): A list with all the features
    """

    def parse_content(self, content):
        self.features = []
        for line in get_active_lines(content):
            self.features.extend(line.split())



"""
freeipa_healthcheck_log - File ``/var/log/ipa/healthcheck/healthcheck.log``
===========================================================================

This module provides plugins access to file ``/var/log/ipa/healthcheck/healthcheck.log``

This file contains a list of items, each item representing a check made on the local IPA server.
The file can be either single-line or multi-line indented.

Typical content of file ``/var/log/ipa/healthcheck/healthcheck.log`` is::

    [
      {
        "source": "ipahealthcheck.ipa.roles",
        "check": "IPACRLManagerCheck",
        "result": "SUCCESS",
        "uuid": "1f4177a4-0ddb-4e4d-8258-a5cd5f4638fc",
        "when": "20191203122317Z",
        "duration": "0.002254",
        "kw": {
          "key": "crl_manager",
          "crlgen_enabled": true
        }
      },
      {
        "source": "ipahealthcheck.ipa.roles",
        "check": "IPARenewalMasterCheck",
        "result": "SUCCESS",
        "uuid": "1feb7f99-2e98-4e37-bb52-686896972022",
        "when": "20191203122317Z",
        "duration": "0.018330",
        "kw": {
          "key": "renewal_master",
          "master": true
        }
      },
      {
        "source": "ipahealthcheck.system.filesystemspace",
        "check": "FileSystemSpaceCheck",
        "result": "ERROR",
        "uuid": "90ed8765-6ad7-425c-abbd-b07a652649cb",
        "when": "20191203122221Z",
        "duration": "0.000474",
        "kw": {
          "msg": "/var/log/audit/: free space under threshold: 14 MiB < 512 MiB",
          "store": "/var/log/audit/",
          "free_space": 14,
          "threshold": 512
        }
      }
    ]

The list of errors can be accessed via the issues property.

Examples:

    >>> len(healthcheck.issues)
    1
    >>> healthcheck.issues[0]['check'] == 'FileSystemSpaceCheck'
    True

"""

from insights import parser
from insights import JSONParser
from insights.specs import Specs


@parser(Specs.freeipa_healthcheck_log)
class FreeIPAHealthCheckLog(JSONParser):
    """Parses the content of file ``/var/log/ipa/healthcheck/healthcheck.log``."""

    @property
    def issues(self):
        """ list: non-success results in healthcheck log."""
        return [entry for entry in self.data if entry["result"] != "SUCCESS"]

    def get_results(self, source, check):
        """Given a source and check find and return the result"""
        results = []
        for entry in self.data:
            if (entry.get('source') == source and entry.get('check') == check and entry.get('result') in ['ERROR', 'CRITICAL']):
                results.append(entry)

        return results



try:
    import cjson
    json_encode = cjson.encode
    json_decode = cjson.decode
except ImportError:
    import json
    json_encode = json.dumps
    json_decode = json.loads

import six


class Marshaller(object):
    """
    Marshalling class that restructures parser output
    for use in the reduce phase.
    """

    def marshal(self, o, use_value_list=False):
        """
        Packages the return from a parser for easy use in a rule.
        """

        if o is None:
            return
        elif isinstance(o, dict):
            if use_value_list:
                for k, v in o.items():
                    o[k] = [v]
            return o
        elif isinstance(o, six.string_types):
            if use_value_list:
                return {o: [True]}
            else:
                return {o: True}
        else:
            raise TypeError("Marshaller doesn't support given type %s" % type(o))

    def unmarshal(self, doc):
        return doc


class JsonMarshaller(Marshaller):
    """
    Marshalling class that json encodes/decodes
    """

    def marshal(self, o, use_value_list=False):
        return json_encode(super(JsonMarshaller, self).marshal(o, use_value_list))

    def unmarshal(self, doc):
        return json_decode(doc)


INSTANCE = JsonMarshaller()
marshal = INSTANCE.marshal
unmarshal = INSTANCE.unmarshal
